C Copyright, Dr. H. K. D. H. Bhadeshia, University of Cambridge
C  and Kazutoshi Ichikawa, Nippon Steel
C Department of Materials Science and Metallurgy, Pembroke St.
C Cambridge CB2 3QZ, England. Telephone Cambridge 334301.
C
C Major modifications March 1994
C Major modifications 5th July 1994
C Modification to include variable cooling rate, 19 October 1994
C Description of input data about welding parameters were also added to be
C appeared on the screen by K. Ichikawa on 19 October 1994
C Version 1.2
C
C Major modifications 
C Modified to take acount the segregation effect by K. Ichikawa on
C 1 Nov., 1994
C
C This program is based on a rectangular segregation model.
C Adding up of VF is not possible by this program for each calculating Temp.
C 5 JAN 1995.
C
C Interpolation method is adapted for adding up the VF.
C K. ICHIKAWA on 5 JAN 1995.
C
C Allows carbon enrichment in austenite to affect the kinetics of
C  ferrite formation via soft impingement effects
C
C Program on overall transformation kinetics (non-isothermal).
C
C Subroutine NUC designed from their nucleation function.
C
C needs to ensure that ANS, AREA increase as T decreases. 
C this is a problem associated with the treatment of anisothermal 
C transformation.
C There should never be a decrease in the fraction of boundary covered
C as the temperature drops.
C
C TYPICAL DATA SET FOLLOW
C 0.003 8392
C 760 610  2D04 0.217 0.05 1.0
C C MOL FRAC, carbon-carbon interaction energy, J/mol,
C  approx Ae3 centigrade,
C Approx allot. ferrite finish temp centigrade,
C surface per unit volume
C DUMM, FIDDLE and SITE are required for subroutine NUC
C 0.0139  -112.0
C Equilibrium C content of austenite
C Maximum driving force for diffusional nucleation, J/mol
C End of dataset
C
C AB1 is set between 0,1,saturation value for grain boundary coverage
C AB2 sets the temperature interval between successive calculations
C   of parabolic rate constant
C CARBON, SI, MN, NI, MO, CR, V in wt.%
C
C SITE allows the number of sites for nucleation to be multiplied by a factor
C SIGMA is the interface energy per unit area
C SIGMA2 is an effective interface energy which takes into account the
C   effect of DUMM
C LINEAL is the mean lineal intercept in micrometers for equiaxed grains
C SV is the austenite grain surface per unit volume (1/m)
C
C TIME2 is the time to achieve the same fraction at a lower temperature
C TIME3 is the net time spent at any given temperature to achieve
C   the total amount of transformation that has occurred
C TIME(I) is defined to be zero at the Ae3 temp,
C   and it is the TOTAL amount of time between Ae3 and the temperature
C   of interest. It is incremented in regular steps determined by the
C   cooling curve.
C
C XBAR mole fraction carbon in alloy as a whole
C XBARN mole fraction carbon in enriched austenite
C
C Structure of the program has completely revised on 21 Dec., 1994
C by K.Ichikawa. That is into as followings.
C (1) Mainprogram only manages the datainput, output for each region 
C     (i.e., Enriched region, Depleted region, ...(Number of Region
C     can be increased as any Segregation profile model)), calculations
C     of adding up the VF for all regions and their output.  
C (2) "Main" calculations of transformation are conducted in a Sub- 
C     routine "MAINCONP", which is almost same as original program by 
C     Dr. Bhadeshia.
C
C Sheil equation was introduced to provide segregation profiles for  
C each element on 12 Jan. 1995 by K.Ichikawa.
C
C Adding up of VF for each divided segment regarding to the 
C segregations is still impossible on 12 Jan. by K. Ichikawa.  
C
C The number of division "n" into the independed segment should be 
C as following.
C                       (1/n) < Maximum VF
C
C Therefore (1/n) < VF in the most depleted region. 
C Subroutine DETN calculate the "n" according to the above principle.
C K. Ichiokawa on 20 Jan., 1995.
C
C
C New cooling rate data on (very) large heat input were installed
C by K. Ichikawa on 2 May, 1995 in SUBROUTINE HFLOW
C
C In this program, hard impingement effect was introduced in the 
C calculation of Widmanstatten ferrite of ferrp48.for  
C by K. Ichikawa on 19 July 1995.
C
C Trying to find Widmanstatten Ferrite Transformation Start Temp. in  
C more regorous way. By K. Ichikawa on 12 Oct 1995.
C
C Harry's new concept on simultaneous kinetics is being adapted in this
C program.
C
C Hard inpingement effects are temporarily killed in this program 
C (6 Feb. 1996).  
C
C SUBROUTINE PROGC were replace to the original version by Harry
C on 15 Feb. 1996. (previously ferrp54.for)
C
C Allow simultaneous transformation of allotriomorphic ferrite and 
C Widmanstatten ferrite. K. Ichikawa 11 April 1996.
C ###################################################################
C This program is completed version on 22th July 1996. 
C ###################################################################
C
C*********************************************************************
      IMPLICIT REAL*8(A-H,K,M,O-Z,N,L), INTEGER(I,J)
      DOUBLE PRECISION C(8),CTEMP(100),CTEMPE(200),VWSE(200)
      DOUBLE PRECISION CTEMPW(1000),CTEMPWSE(200),CTEMPWS(200,200)
      DOUBLE PRECISION VFA(200),VFAW(200),VFBAF(200),VFEA(200)
      DOUBLE PRECISION VFEAW(200),VFEBAF(200),VFEW(200),VFERGM(200) 
      DOUBLE PRECISION VFRGM(200),VFW(200),VW(1000),VWS(200,200)  
C
      FIDDLE=1.0D+00
      SIGMA=0.05D+00
      AB1=0.90D+00
      AB2=5.0
      ICI=0
      IIMP=0
C
C ICI is a indicater which indicates the incorrect convergent of
C calculation of XEQ, GMAX. IF ICI=0, no such probrem was occured in   
C the calculation. IF ICI=2 convergent could NOT completed in the 
C calculation in SUBROUTINE AG and/or HIGH.
C
      WRITE(*,*) 'Input C   Si   Mn   Ni   Mo   Cr   V (wt%)'
      READ(*,*) CARBON,SI,MN,NI,MO,CR,V
      WRITE(*,*) 'Input austenite grain surface per unit volume (1/m):'
      READ(*,*) SV
      WRITE(*,*) 'Input fitting factor for nucleation rate (K2):'
      READ(*,*) DUMM
      WRITE(*,*) 'Input fitting factor for number of nucleation ',
     &           'sites (K1):'
      READ(*,*) SITE
      WRITE(*,*) 'Input lowest temperature for calculations',
     &          ' (degrees C):'
      READ(*,*) LOWT
C
C Read welding parameters
C J defines the welding process (see subroutine HFLOW)
C CURR is the welding current, amps
C VOLT is the welding voltage, Volts
C SPEED is the welding speed, m/s
C TINT is the interpass temperature, Centigrade
C
      WRITE(*,*) 'Input welding process:'
      WRITE(*,*) '1 - SMAW; 2 - metal core wire CO2 shielding;',
     & '3 - MCW Fogon 20 shielding; 4 - tandem SAW; 5 - single SAW'
      READ(*,*) J
      WRITE(*,*) 'Input welding current (Amps):'
      READ(*,*) CURR
      WRITE(*,*) 'Input welding voltage (Volts):'
      READ(*,*) VOLT
      WRITE(*,*) 'Input welding speed (m/s):'
      READ(*,*) SPEED
      WRITE(*,*) 'Input interpass temperature (degrees C):'
      READ(*,*) TINT
      WRITE(*,7) CURR,VOLT,SPEED,J,TINT
C
      C(1)=CARBON
      C(2)=SI
      C(3)=MN
      C(4)=NI
      C(5)=MO
      C(6)=CR
      C(7)=V
C       
      CALL INITCC(C,CARBON,SI,MN,NI,MO,CR,V)
C
C Display of Average Chemical Composition as a reference
C
      WRITE(*,446)
      WRITE(*,445)
      WRITE(*,261) (C(J5),J5=1,7)
      CALL OMEGA(C,W,XBAR,T10,T20,0)
      XBARN=XBAR
      WRITE(*,262) (C(J5),J5=1,7)
      CALL INITCC(C,CARBON,SI,MN,NI,MO,CR,V)
C
C       Explanation of the Process of Calculation:
C
C**************************************************************
C Modificated using Sheil equation by K. Ichikawa
C on 9 Jan. ,1995.
 
      CALL DETN(C,SV,DUMM,SITE,LOWT,J,CURR,VOLT,SPEED,TINT,CTEMP,  
     &   VFAW,VFA,VFW,VFBAF,VFRGM,J2,HIGHT,J10,CTEMPW,IIMP,ISTOP)     
      DO 80 I=1,J10
         CTEMPE(I)=0.0
         VFEAW(I)=0.0
         VFEA(I)=0.0
         VFEW(I)=0.0
         VFEBAF(I)=0.0
         VFERGM(I)=0.0       
   80    CONTINUE
C
      DO 50 I=1,J10,1
C 
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% v
C IIMP is 1, if the transformation never occurred because of too 
C low WS Temp. Once IIMP becomes 1, calculations of transformations for 
C following segments are not done. So, the following MAINCONP is
C being skipped.       
C
C        IF (IIMP .EQ. 1) THEN
C           J2=1
C           CTEMP(1)=0.0
C           VF(1)=0.0
C           GOTO 51
C        ENDIF
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ^
C
         WRITE (*,570)
         CALL INITCC(C,CARBON,SI,MN,NI,MO,CR,V)
         WRITE(*,560) I, J10, 1.0*I/J10
         CALL SCHEIL(CARBON,SI,MN,NI,MO,CR,V,C,I,J10)
         CALL MAINCONP(C,SV, DUMM,SITE,LOWT,J,CURR,VOLT,SPEED,TINT,  
     &                 CTEMP,VFAW,VFA,VFW,VFBAF,VFRGM,J2,HIGHT,ICI,
     &                 CARBON,SI,MN,NI,MO,CR,V,VW,CTEMPW,IIMP,ISTOP)
   51 CONTINUE
C
C %%%%%%%%%%%%% Memory Over Flow? %%%%%%%%%%%%% v
C        DO 60 J7=1,J2
C           CTEMPS(I,J7)=CTEMP(J7)
C           VFSAW(I,J7)=VFAW(J7)
C           VFSA(I,J7)=VFA(J7)
C           VFSW(I,J7)=VFW(J7)
C           VFSBAF(I,J7)=VFBAF(J7)
C           VFSRGM(I,J7)=VFRGM(J7)
C   60   CONTINUE
C 
C        CALL INITCC(C,CARBON,SI,MN,NI,MO,CR,V) 
C        CTEMPE(I)=CTEMPS(I,ISTOP)
C        VFEAW(I)=VFSAW(I,ISTOP)
C        VFEA(I)=VFSA(I,ISTOP)
C        VFEW(I)=VFSW(I,ISTOP)
C        VFEBAF(I)=VFSBAF(I,ISTOP)
C        VFERGM(I)=VFSRGM(I,ISTOP)
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ^
C
         CALL INITCC(C,CARBON,SI,MN,NI,MO,CR,V) 
         CTEMPE(I)=CTEMP(ISTOP)
         VFEAW(I)=VFAW(ISTOP)
         VFEA(I)=VFA(ISTOP)
         VFEW(I)=VFW(ISTOP)
         VFEBAF(I)=VFBAF(ISTOP)
         VFERGM(I)=VFRGM(ISTOP)
         CTEMPWSE(I)=CTEMPWS(I,J9)
         VWSE(I)=VWS(I,J9)
   50 CONTINUE    
      CALL SUMF(CTEMPE,VFEAW,VFEA,VFEW,VFEBAF,VFERGM,J10,1)
      IF (ICI .EQ. 2) WRITE(*,580)
      STOP 
  580 FORMAT(5X,'*** Calculations of XEQ and/or GMAX could not be'/
     &      10X,' completed correctly for some "i" ***'//)
    7 FORMAT(//20X,'Welding Current (amps) =',F5.0/
     &20X,'Voltage (V) =',F3.0/
     &20X,'Welding Speed (m/s) =',D10.3/
     &20X,'Welding Technique (SMAW:1, Tandem SAW:4, Single SAW:5)=',I5/
     &20X,'Interpass Temperature (Centigrade) =',F5.0)
  446 FORMAT('--------------------------------------------------',
     &       '--------------------------------',
     &       '------------------------------------')
  445 FORMAT (4H    )
  261 FORMAT (5H   C=,F8.4,5H  Si=,F8.4,5H  Mn=,F8.4,
     &        5H  Ni=,F8.4,5H  Mo=,F8.4,5H  Cr=,F8.4,5H   V=,F8.4,
     &        5X,'wt.%')
  262 FORMAT (5H   C=,F8.4,5H  Si=,F8.4,5H  Mn=,F8.4,
     &        5H  Ni=,F8.4,5H  Mo=,F8.4,5H  Cr=,F8.4,5H   V=,F8.4,
     &        5X,'mole fraction'/)      
  550 FORMAT (/'Coefficients A(1), A(2) and A(3) of quadaratic',
     &    ' polynomial y=A(1)+A(2)x+A(3)x**2 for Segregated Region.'/)
  510 FORMAT (/'Coefficients A(1), A(2) and A(3) of quadaratic',
     &      ' polynomial y=A(1)+A(2)x+A(3)x**2 for Depleted Region.'/)
  520 FORMAT ('Where y is VF and x is CTEMP.')
  560 FORMAT(//20X,'i =',I5/20X,'n =',I5/20X,'i/n =',F8.4)
  570 FORMAT('=================================================',
     &       '==================================================',
     &       '==================')
      END
C
C ***********************************************************************
C
      SUBROUTINE MAINCONP(C,SV,DUMM,SITE,LOWT,J,CURR,VOLT,SPEED,  
     &           TINT,CTEMP,MVF,MVFA,MVFW,MVFBAF,MVFRGM,J3,HIGHT,ICI,
     &           CARBON,SI,MN,NI,MO,CR,V,VW,CTEMPW,IIMP,ISTOP)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION ALP(2000),C(8),CTEMP(2000),G(1000),MVF(2000)
      DOUBLE PRECISION GV(2000),MVFA(2000),MVFW(2000),MVFBAF(2000)
      DOUBLE PRECISION MVFRGM(2000),TIME(2000),TIMED(1000)
      DOUBLE PRECISION TIMEWS(2000),TSTIME(1000),TTIME(1000),VFW(2000)
      DOUBLE PRECISION TIMEA(2000),TIMEII(1000),VOLFA(2000),VOLF(2000)
      DOUBLE PRECISION XAGA(2000),XGAG(2000)
C
C Arranged by K. Ichikawa, University of Cambridge based on the Harry's
C main program; "ferrp". 
C
      FIDDLE=1.0D+00
      SIGMA=0.05D+00
      AB1=0.90D+00
      AB2=5.0D+00
      IMPING=0
      ISTOPIMP=0
      IEND=0
C
      WRITE(*,446)
      WRITE(*,445)
      WRITE(*,261) (C(J5),J5=1,7)
      CALL OMEGA(C,W,XBAR,T10,T20,0)
      XBARN=XBAR
      WRITE(*,262) (C(J5),J5=1,7)
C     WRITE (*,28) T10,T20,W
      CALL HIGH(C,W,T10,T20,HIGHT,ICI)
      IF (HIGHT .LE. LOWT) GOTO 2
C
      SIGMA2=SIGMA*(DUMM**0.33)
      LINEAL=(2.0D+00/SV)*1.0d+06
      WRITE(*,7) SIGMA,SIGMA2, HIGHT,LOWT, SITE,SV,LINEAL,
     &           FIDDLE,AB1
      WRITE(*,446)
      J3=((HIGHT-LOWT)/AB2)+1
      HIT=HIGHT+AB2
C
C Reset matrix
C
      DO 4 I=1,J3
         TIME(I)=0.0
         TIMEA(I)=0.0
         VOLF(I)=0.0
         VOLFA(I)=0.0
         VFW(I)=0.0
         G(I)=0.0
         CTEMP(I)=0.0
         MVF(I)=0.0
         MVFA(I)=0.0
         MVFW(I)=0.0
         MVFBAF(I)=0.0
         MVFRGM(I)=0.0
         TIMEWS(I)=0.0
    4  CONTINUE
C
C Reset parameters
C                    
       WRITE(*,*)
       DUMMY=DUMM
       IF(DUMMY .LT. 0.0D+00) GOTO 2
       TIME2=0.0D+00
       TIME3=0.0D+00
       TIME2A=0.0D+00
       TIME3A=0.0D+00
       JTEST=0
       AV=0.0
       AVA=0.0
C
C                      ***** Calculations for each cooling rate
C       
       IWC=1
       IWS=1
       ILC=1
C
C########################### v
       WSI=0.0D+00
       VMAX=0.0+00
C########################### ^
C
       DO 1 I=1,J3
          ISTOP=J3
C
C calculate grain boundary nucleation rate
C
          CTEMP(I)=HIT-AB2*I
          KTEMP=CTEMP(I)+273.15D+00
          CALL AG(C,W,XBARN,T10,T20,KTEMP,XGAG(I),GMAX,FPRO,
     &            H,S,H1,S1,W1,F,AJ,AJ1)
C
C Following "CALL AGX" was put for the calculation of AXTO.
C This "AGX is almost same as "SUBROUTINE AG".
C 25 June 1996. 
C
C         CALL AG(C,WX,XBARN,T10X,T20X,KTEMP,XGAGX(I),GMAXX,FPROX,
C     &           HX,SX,H1X,S1X,W1X,FX,AJX,AJ1X)
C 
          XAGA(I)=XALPH(CTEMP(I))
          CALL ALP11(W,XBARN,CTEMP(I),XAGA(I),XGAG(I),ALP(I))
          ALP(I)=ALP(I)/1.0D+02
          GV(I)=GMAX/7.2D-06
          XAGA1=XAGA(1)
          XGAG1=XGAG(1)       
C
          CALL TIM(CULRAT1,TIMEI,AB2,CTEMP(I),CURR,VOLT,SPEED,TINT,J)
C
C ############################ v
          TIMEII(I)=TIMEI
C ############################ ^
C
          IF(I .EQ. 1) THEN
             TIME(I)=TIMEI
             TIMEA(I)=TIMEI
          ELSE
             TIME(I)=TIMEI+TIME(I-1)
             TIMEA(I)=TIMEI+TIMEA(I-1)
          ENDIF
C
          CALL NUC(SITE,GSTAR,CTEMP(I),GV(I),BI,DUMMY,FIDDLE,SIGMA)
C
C calculate fraction of ferrite, for aniosthermal transformation
C using the Scheil rule, beginning with the highest temperature (I=1)
C
          IF (I .EQ. 1) THEN
             CALL AVOLFW2(AREA,ANS,VF,SV,BI,TIME(I),ALP(I),
     &                    VOLF(I),XBAR,XAGA(I),XGAG(I),WSI,VMAX,XBAR,
     &                    XAGA1,XGAG1)
             CALL AVOLFW1(AREAA,ANSA,VFA,SV,BI,TIMEA(I),ALP(I),
     &                    VOLFA(I),XBAR,XAGA(I),XGAG(I),WSI,VMAX,XBAR,
     &                    XAGA1,XGAG1)
          ELSE
C
C allow for carbon enrichment in austenite by 
C allotriomorphic and Widmanstatten ferrite transformations
C
C VFW is not necessary any more so that following 4 lines are
C killed here. 
C               IF(I .EQ. 1) THEN
C               XBARN=(XBAR - XAGA(I)*(VF))/(1.0-V)
C               ELSE
C               XBARN=(XBAR - XAGA(I)*(VF+VFW(I-1)))/(1.0-VF-VFW(I-1))
C               ENDIF
C Following new line is installed.
C
         XBARN=(XBAR - XAGA(I)*VF)/(1.0D+00-VF)
C 
C ########################### v
C Impingement effect. 
C Condition: If The Temp. < T_0,
C then all the transformation stop.
C 
C 6 June 1996
C
C Modefied 25 June 1996
C
C       
         CALL AXTO(H,S,H1,S1,XTO,KTEMP,W,W1,F,AJ,AJ1,STORE,XBARN,XBAR)
     &             
         IF (IWC .GE. 2) THEN
            TIMEWS(I)=TIME(I)-TIME(IWS)
         ENDIF
C        WRITE(*,*)'########################## Begining of ', 
C    &     'the next temp step ##########################' 
C        WRITE(*,*)'X_T_zero (mole fraction) =',XTO
C        WRITE(*,*)'C in Austenite (mole fraction) =',XBARN
C        WRITE(*,*)'Time from alpha_W start /sec =',TIMEWS(I)
C        WRITE(*,*)'OMEGAO=',(XGAG(I)-XBAR)/(XGAG(I)-XAGA(I))
C
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% v
C 
         IF (XBARN .LE. XTO) THEN 
C           WRITE(*,*)
C 
C     &'!!alpha & alpha_W were impinged at the following carbon
C     & concentrations.!!'
C       
C IIMP is a "flag". It becomes 1 when the Widmanstatten Ferrite 
C Transformation Starting Temp. is too low and the transformation
C does not occur.
C 
            IF (WS .LT. CTEMP(I)) THEN
               IIMP=1          
            ENDIF
C
C ISTOPIMP is 1 when the impingement occured with the acicular 
C ferrite. 
C
C ISTOP is value of I at which final VF can be seen for the   
C segregation segment. 
C
C If ISTOPIMP=1, ISTOP=I-1. Because at this I here, VF=0.0, 
C because the VF calculation were skipped. But the other case 
C ISTOP should be I.
C
C Modified after running the runallcw on 18 June 1996.
C           
           ISTOP=I-1
           ISTOPIMP=1
           GOTO 700
C
C Getting out from the do-loop. (DO-loop of "DO 1 I=... .")
C
         ENDIF
C
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ^
C        
C ############################ ^
C
    3    AV=VOLF(I-1)
         AVA=VOLFA(I-1)
         IF(AV .EQ. 1.0D+00) GOTO 2
         TIME2=TIME(I-1)
         TIME2A=TIMEA(I-1)
         CALL ATIM(SV,BI,TIME2,ALP(I),AV,XBARN,XAGA(I),XGAG(I),
     &             VMAX,WSI,XBAR,XAGA1,XGAG1)
         CALL ATIMA(SV,BI,TIME2A,ALP(I),AVA,XBARN,XAGA(I),XGAG(I),
     &              VMAX,WSI,XBAR,XAGA1,XGAG1)

         TIME3=TIME(I)+TIME2-TIME(I-1)
         TIME3A=TIMEA(I)+TIME2A-TIMEA(I-1)
C
C ########################## v
C Under modification
C
         CALL AVOLFW2(AREA,ANS,VF,SV,BI,TIME3,ALP(I),VOLF(I),XBARN,
     &                XAGA(I),XGAG(I),WSI,VMAX,XBAR,XAGA1,XGAG1)       
         CALL AVOLFW1(AREAA,ANSA,VFA,SV,BI,TIME3A,ALP(I),VOLFA(I),
     &                XBARN,XAGA(I),XGAG(I),WSI,VMAX,XBAR,XAGA1,XGAG1)         
C
C ########################## ^
C
         ENDIF
         MVF(I)=VF
         MVFA(I)=VFA
         IF (AREA .GE. 0.005) THEN
C           WRITE(*,5) CTEMP(I),VOLF(I),MVF(I),
C    &              AREA,ANS,GV(I),GSTAR,XAGA(I),XGAG(I),
C    &              TIME(I),TIME3,TIME2,ALP(I),BI,CULRAT1
C           WRITE(*,*)VOLFA(I),'   ',MVFA(I)
            IF (VOLF(I) .GE. 0.99) GOTO 2
         ENDIF
C
C  check for site saturation
C  jtest ensures grain size calculation at site saturation
C  jtest set to unity at site saturation
C
         IF (AREA .GT. AB1 .AND. JTEST .EQ. 0) THEN
            CALL ISONUC(TIME3,SV,BI,ALP(I),XBARN,XAGA(I),XGAG(I),AB1
     &                  ,WSI,VMAX,XBAR,XAGA1,XGAG1)
            CALL ISONUCA(TIME3A,SV,BI,ALP(I),XBARN,XAGA(I),XGAG(I),AB1
     &                   ,WSI,VMAX,XBAR,XAGA1,XGAG1)
            JTEST=1
         ENDIF
C
C Under Modification, being installed the Widmanstatten
C Ferrite Transformation. 
C
         CALL MOLTOWT(XBARN,C,CARBON,SI,MN,NI,MO,CR,V)
C        WRITE(*,261) (C(J5),J5=1,7)
         IF (IWC .LE. 1) THEN 
            CALL WSTEMP(C,W,GAX,GAAF,WS,HIGHT)
            WS=WS-273.15D+00
C
C WS is Widmanstatten Ferrite Transformation Starting Temperature    
C in degree C.
C        
C           WRITE(*,1005) WS
         ENDIF       
         IF (IWC .GE. 2) THEN 
            CALL OMEGA(C,W,XBARN,T10W,T20W,0)
         ENDIF    
 1005 FORMAT(' Widmanstatten Ferrite starts at, (degree C) ',F8.1)
C
         CTEMPW=CTEMP(I)
C
         IF (CTEMP(I) .LE. WS) THEN         
            IWC=IWC+1
            IF (IWC .EQ. 2) THEN
               IWS=I
            ENDIF         
            IF (IMPING .EQ. 1) THEN 
               GOTO 610
            ENDIF       
            CALL VWMAINCOMP(SV,CURR,VOLT,EFF,SPEED,TINT,
     &           T,XMAX,WDIFF,W,XBARN,AB2,VF,J2,CTEMP,J,C,T10,T20,
     &           J3,VFW,CTEMPW,HIGHT,I,WS,JW,TIMED,TTIME,G,VMAX,WSI)
C
C ########################## v
C
            IF (CTEMP(I) .GT. WS) THEN 
               WSI=0.0D+00
               VMAX=0.0D+00
            ENDIF
C
C ########################## ^
C
            IF (I .EQ. 1) THEN
               TTIME(I)=TIMED(I)
            ELSE
               TTIME(I)=TTIME(I-1)+TIMED(I)
            ENDIF         
            IF (IWC .EQ. 2) THEN
               TSTIME(I)=0.0
            ENDIF
            IF (IWC .GE. 3) THEN
               TSTIME(I)=TSTIME(I-1)+TIMEII(I)
            ENDIF 
C
C TIMEW(I) is Times in the SUBROUTINE VWMAINCOMP. This times are 
C introduced here to store all the values of TIME(I) (for I=1, 2.
C 3, ...) in the SUBROUTINE VWMAINCONP. 
C                 
         ENDIF
   60    FORMAT(//'Widmanstatten Ferrite was impinged with', 
     &            'Allotriomorphic Ferrite'/)         
   80    FORMAT(//'Widmanstatten Ferrite was impinged with', 
     &            'Acicular Ferrite'/)
   81    FORMAT(//'All the transformations were impinged'/)      
   82    FORMAT(F5.0)
C
         ILC=ILC+1
  610    CONTINUE
  620    CONTINUE
C
C %%%%%%%%%%%%%%%%%%%%%%%%% v
C 
         IF (I .EQ. J3) THEN
            IEND=1
         ENDIF 
C
C %%%%%%%%%%%%%%%%%%%%%%%%% ^
C
    1 CONTINUE
  700 CONTINUE
C %%%%%%%%%%%%%%%%%%%%%%%%%
C Display again the carbon concentration when the impingement 
C with the acicular ferrite occured. 
C      WRITE(*,*)'X_T_zero (mole fraction) =',XTO
C      WRITE(*,*)'C in Austenite (mole fraction) =',XBARN
C      WRITE(*,*)'Time from alpha_W start /sec =',TIMEWS(I)
C %%%%%%%%%%%%%%%%%%%%%%%%%
C Display the amount of Widmanstatten ferrite transformed.
C by K. Ichikawa on 4 Dec., 1995. 
C
C      WRITE(6,600)
C   2 OPEN(10,FILE='widtip')
    2 CONTINUE
      DO 1000 I=1,J3         
         IF (CTEMP(I) .EQ. 0) THEN
            GOTO 1020
         ENDIF    
C        WRITE(10,601)CTEMP(I),MVF(I)  
 1000 CONTINUE
 1020 CONTINUE
C     WRITE(10,*)'0','    ','0'
C     WRITE(10,1010)
      DO 500 I=1, J3
C
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% v
C
C Following two lines were modified for the case of that calculation 
C is stopped because of VOLF( )=1.0
C on 25 July 1996. 
C  
         IF (CTEMP(I) .EQ. 0.0) THEN 
            ISTOP=I-1
         ENDIF
C         
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ^          
C
         MVFW(I)=MVF(I)-MVFA(I)  
C        WRITE(*,601)CTEMP(I),MVF(I),MVFA(I),MVFW(I),TIMEWS(I)        
C
C Following two lines were modified for the case of that calculation 
C is stopped because of VOLF( )=1.0
C on 25 July 1996.
C          
         IF (I .GE. ISTOP) THEN
            J4=ISTOP
            GOTO 510
         ENDIF 
  601    FORMAT(' ',F5.0,'  ',F7.5,'   ',F7.5,'   ',F7.5,'   ',F7.5)
  600    FORMAT(//' CTEMP VF_alpha+W VF_alpha  VF_W      TIME_W'/)      
  500 CONTINUE
      GOTO 520
  510 CONTINUE
C     WRITE(*,*)' '
      IF (ISTOPIMP .EQ. 1) THEN 
C        WRITE(6,*)'Fraction of bainite+AF=',1.00D+00-MVF(J4)
C        WRITE(6,*)'Retained Austenite+M= 0.00000'         
         MVFBAF(ISTOP)=1.00D+00-MVF(J4)
         MVFRGM(ISTOP)=0.0
      ELSE
C        WRITE(6,*)'Fraction of bainite+AF= 0.00000'
C        WRITE(6,*)'Retained Austenite+M=',1.00D+00-MVF(J4)
         MVFBAF(ISTOP)=0.0
         MVFRGM(ISTOP)=1.00D+00-MVF(J4)
      ENDIF
      GOTO 530
  520 CONTINUE
      WRITE(*,*)' '
      IF (ISTOPIMP .EQ. 1) THEN
C        WRITE(6,*)'Fraction of bainite+AF=',1.00D+00-MVF(J3)
         MVFBAF(ISTOP)=1.00D+00-MVF(J3)
         MVFRGM(ISTOP)=0.0
      ELSE
C        WRITE(6,*)'Retained Austenite+M=',1.00D+00-MVF(J3)
         MVFBAF(ISTOP)=0.0
         MVFRGM(ISTOP)=1.00D+00-MVF(J3)
      ENDIF
  530 CONTINUE
 1010 FORMAT('     ')
      RETURN
    5  FORMAT(F5.0,4F7.2,2D10.2,D10.2,F10.4,3F8.2,2D9.2,D10.3,D10.2)
    6 FORMAT(//' CTEMP   VOL    VF   AREA     ANS    GV     GSTAR     ',
     &'  XAGA      ',
     &' XGAG   TIME(I)  TIME3   TIME2 ',
     &'   ALP(I)    BI     K/s '/)
    7 FORMAT(/20X,'Interface energy,           J/m**2= ',F10.3/
     &20X,         'Effective interface energy, J/m**2= ',F10.3//
     &20X,'Paraequilibrium Ae3 (Centigrade)',F5.0/
     &20X,'Temperature transformation stopped (Centigrade)',F5.0/
     &20X,'Fraction of boundary atoms forming nucleation sites',D10.3//
     &20X,'Austenite grain surface per unit volume     SV (1/m)',D10.3/
     &20X,'Lineal intercept, equiaxed austenite grains (microns)',F10.0//
     &20X,'Fiddle (1 means no fiddle factor) =',D12.4/
     &20X,'Nucleation stopped at fractional coverage = ',F8.3//)
   28 FORMAT (/'    T10=',F10.6,
     &'     T20=',F10.6,'     WGAMMA=',F7.0,'  J/mol')
  446 FORMAT('--------------------------------------------------',
     &       '--------------------------------',
     &       '------------------------------------')
  445 FORMAT (4H    )
  261 FORMAT (5H   C=,F8.4,5H  Si=,F8.4,5H  Mn=,F8.4,
     &        5H  Ni=,F8.4,5H  Mo=,F8.4,5H  Cr=,F8.4,5H   V=,F8.4,
     &        5X,'wt.%')
  262 FORMAT (5H   C=,F8.4,5H  Si=,F8.4,5H  Mn=,F8.4,
     &        5H  Ni=,F8.4,5H  Mo=,F8.4,5H  Cr=,F8.4,5H   V=,F8.4,
     &        5X,'mole fraction'/)
       END
C
C************************************************************************
C
      SUBROUTINE AARG(ARG)
C
C HELPS AVOID UNDERFLOW ERRORS IN EXPONENTIALS
C
      DOUBLE PRECISION ARG
      IF(ARG .GT. 173.0D+00) GOTO 1
      GOTO 2
    1 ARG=179.0D+00
    2 RETURN
      END
C
C*****************************************************************
C
C C AFEG AFEG LN ACTIVITY OF IRON IN GAMMA
C
      DOUBLE PRECISION FUNCTION AFEG(XEQ,AJ)
C
      DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      TEQ=5*DLOG((1-XEQ)/(1-2*XEQ))
      TEQ=TEQ+DLOG(((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1)))**6)
      AFEG=TEQ
      RETURN
      END
C
C*****************************************************************
C
C Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
C MUCG1.FOR
C calculation of XEQ and GMAX only. Adapted from MUCG46.FOR
C 5 July 1994
C
C Department of Materials Science and Metallurgy, Pembroke St. Cambridge CB2 3QZ
C
      SUBROUTINE AG(C,W,XBAR,T10,T20,KTEMP,XEQ,GMAX,FPRO,
     &              H,S,H1,S1,W1,F,AJ,AJ1)
C
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8)
C
      X1=C(1)
      XA=0.001D+00
      R=8.31432D+00
      W1=48570.0D+00
      H=38575.0D+00
      S=13.48D+00
      XEQ=0.1D+00
      IC=0
C
      T=KTEMP
      CTEMP=T-273.00D+00
C
      IF (T .LE. 1000.0) GOTO 20
      H1=105525.0D+00
      S1=45.34521D+00
      GOTO 19
   20 H1=111918.0D+00
      S1=51.44D+00
C
   19 F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      AJ1=1.0D+00-DEXP(-W1/(R*T))
   51 TEQ=R*T*AFEG(XEQ,AJ)-F
      IF (DABS(TEQ) .LT. 1.0) GOTO 52
      ETEQ=DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-TEQ/ETEQ
C
C Modified by K. Ichikawa Until ****
C
      IC=IC+1
      IF (IC .GE. 1000000) GOTO 55
     
C ****     
      GOTO 51
   52 AEQ=CG(XEQ,T,W,R)
      AFEQ=AFEG(XEQ,AJ)
      A=CG(X1,T,W,R)
      AFE=AFEG(X1,AJ)
C
C     Following "FPRO" was put according to Harry's direction
C     in order to get free energy change for growth.
C     FPRO is free energy change for ferrite growth.
C
C
      FPRO=R*T*(X1*(AEQ-A)+(1.0-X1)*(AFEQ-AFE))
C
C
C
      CALL GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      GMAX=R*T*(A1-A)
C
   60 RETURN
   55 WRITE(*,56)
      CONTINUE
   56 FORMAT(5X,'*** Calculations of XEQ and GMAX could not'/
     &      10X,' conpleted correctly  ***'//)
      GOTO 60
      END
C
C***********************************************************
C
      SUBROUTINE AL(XGAG,XBAR,XAGA,DIFF,CTEMP,ALPHA)
C
C Writing suppressed 5 July 1994
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION X2(22),Y2(22),X(60),Y(60),X1(3),Y1(3)
C
      GES=0.51466
C     GES IS A GUESS VALUE OF ALPHA1/(2*SQRT(D))
      I3=0
      J=0
      OMEGA=(XGAG-XBAR)/(XGAG-XAGA)
    3 I3=I3+1
      DUM1=GES*DEXP(GES*GES)*(1.0D+00-DERF(GES))-(OMEGA
     &     /DSQRT(3.14159D+00))
      X2(I3)=GES
      Y2(I3)=DUM1
C     WRITE(*,5)GES,DUM1
      DER=(DEXP(GES*GES)*(1.0D+00 + 2.0D+00*GES*GES))*
     &    (1.0D+00-DERF(GES))-GES*2.0D+00/(DSQRT(3.14159D+00))
      I1=DINT(GES*1000000)
      GES=GES-DUM1/DER
      IF(GES .GT. 12.0) GOTO 1
      I2=DINT(GES*1000000)
      IF(I1 .EQ. I2)GOTO 2
      IF(I3 .GT. 20)GOTO 1
      GOTO 3
    1 GES=0.51466
    7 J=J+1
      DUM1=GES*DEXP(GES*GES)*(1.0D+00-DERF(GES))-(OMEGA
     &/DSQRT(3.14159D+00))
      DER=(DEXP(GES*GES)*(1.0D+00 + 2.0D+00*GES*GES))*
     &(1.0D+00-DERF(GES))-GES*2.0D+00/(DSQRT(3.14159D+00))
      X(J)=GES
      Y(J)=DUM1
      IF(J .EQ. 1) GOTO 7
      IF(DABS(Y(J)) .GT. DABS(Y(J-1)))GOTO 6
      GES=GES-0.5*DUM1/DER
      GOTO 7
    6 X1(3)=X(J-1)
      X1(2)=X(J-2)
      X1(1)=X(J-3)
      Y1(3)=Y(J-1)
      Y1(2)=Y(J-2)
      Y1(1)=Y(J-3)
      CALL ANAL(3,0,CONST,SLOPE,CORR,X1,Y1)
      GES=-1.0D+00*CONST/SLOPE
      DUM1=GES*DEXP(GES*GES)*(1.0D+00-DERF(GES))-(OMEGA
     &/DSQRT(3.14159D+00))
      DO 10 J1=1,I3
      IF(DABS(DUM1) .GT. DABS(Y2(J1)))GOTO 9
      GOTO 10
    9 GES=X2(J1)
      DUM1=Y2(J1)
   10 CONTINUE
    2 ALPHA=GES*2.0D+00*DSQRT(DIFF)
C     WRITE(*,4) CTEMP,ALPHA,DUM1
      RETURN
    4 FORMAT(' CTEMP=', F8.0,10X,' ALPHA1, (cm/(sec)**.5)=',
     &       D12.5,   10X,' DUM1=',D12.5/
     & /'******** *********** ********** ************   ***********')
    5 FORMAT(2D12.4)
      END
C
C***********************************************************************
C
      SUBROUTINE ALP11(W,XBAR,CTEMP,XAGA,XGAG,ALPHA)
C
C Modified to suppress writing. 5 July 1994
C
C 27 September 1985 same as .CONCDIF:RATE, BUT modified to
C allow easier low T calculations and provide iteration error messages
C TYPICAL DATASET
C 780 0.0100
C 740 0.0176
C 700 0.0266
C 660 0.0389
C 600 0.0552
C Modified 24-6-85 to include XINCR
C - this limits no. of Diff coeffs calculated for T<700C
C PROGRAM TO CALCULATE PARABOLIC RATE CONSTANT, 4-1-84
C PROGRAM MODIFIED, 30-8-83, FOR ACCURATE XAPLPHA
C PROGRAM TO CALCULATE THE EFFECTIVE DIFFUSIVITY OF CARBON IN
C AUSTENITE, TAKING ACCOUNT OF THE FACT THAT THIS DIFFUSIVITY
C IS CONCENTRATION DEPENDENT.  USES SILLER AND MCLELLAN THEORY
C TO EXPRESS THE CONCENTRATION DEPENDENCE, AND LACHER ET AL
C THEORY TO ALLOW FOR THE EFFECT OF SUBSTITUTIONAL ALLOYING ELEMENTS
C ON THE ACTIVITY AND W OF CARBON IN AUSTENITE. **7 OCTOBER 1981**
C TEMPERATURE IS READ IN IN DEGREES CENTIGRADE
C CARBON CONTENT READ IN AS MOL FRAC, EXCEPT FOR THE SUBROUTINE OMEGA,
C WHERE THE CARBON CONTENT AND ALLOYING CONTENT IS READ IN AS WT PCT IN
C THE FIRST LINE OF THE DATASET.
C HH=PLANCKS CONST.JOULES/SEC,KK=BOLTZMANNS CONST.JOULES/DEGREE KELVIN
C     D=DIFFUSIVITY OF CARBON IN AUSTENITE
C     Z=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      IMPLICIT REAL*8(A-H,K-Y), INTEGER(I,J,Z)
      DOUBLE PRECISION DIFF(500), CARB(500)
C
      HH=6.6262D-34
      KK=1.38062D-23
      Z=12
      A5=1.0D+00
      R=8.31432D+00
      T=CTEMP+273.00D+00
      II2=0
C     WRITE(*,7)T,CTEMP,XBAR,XGAG,XAGA
      DASH=(KK*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
      DO 9   II=1,1000
         CARB(1)=XBAR
         IF (II .GT. 1)GOTO 1
         GOTO 8
    1    IF((XGAG-XBAR) .LT. 0.005)GOTO 2
         GOTO 3
    2    XINCR=0.0001D+00
         GOTO 4
    3    XINCR=0.001D+00
    4    CARB(II)=CARB(II-1)+XINCR
         IF (CARB(II) .GT. XGAG) GOTO 5
    8    X=CARB(II)
         II2=II2+1
         THETA=X/(A5-X)
         ACTIV=CG(X,T,W,R)
         ACTIV=DEXP(ACTIV)
         DACTIV=DCG(X,T,W,R)
         DACTIV=DACTIV*ACTIV
         DACTIV=DACTIV*A5/((A5+THETA)**2)
         SIGMA=A5-DEXP((-(W))/(R*T))
         PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*
     &           (A5+Z/2)*(A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
         DIFF(II)=DASH*PSI
    9 CONTINUE
    5 II3=0
      CALL TRAPE(CARB,DIFF,ANS,II2)
      ANS=ANS/(XGAG-XBAR)
C     WRITE(*,6)ANS,ERROR
      CALL AL(XGAG,XBAR,XAGA,ANS,CTEMP,ALPHA)
      RETURN
    7 FORMAT(' ABSOLUTE TEMPERATURE, DEGREES KELVIN =',F8.1/
     &'           TEMPERATURE IN DEGREES CENTIGRADE =',F8.1/
     &' MOL FRAC CARBON IN ALLOY = ',F8.4/
     &' EQUILIBRIUM MOL FRAC CARBON IN AUSTENITE =',F8.4/
     &' EQUILIBRIUM MOL FRAC OF C IN FERRITE=',D12.4)
    6 FORMAT(' INTEGRAL, XGAG-XBAR = ',D12.4, '  CM**2/SEC  ',
     &8HERROR = , D12.4)
      END
C
C*********************************************************************
C
      SUBROUTINE ANAL(J7,J9,CONST,SLOPE,CORR,X,Y)
C
      DOUBLE PRECISION AX,AX2,AY,AY2,AXY,CONST,CORR,SLOPE
      DOUBLE PRECISION X(60),Y(60)
      INTEGER I,J8,J9,J7
C
      J8=J7-J9
      AX=0.0D+00
      AY=0.0D+00
      AX2=0.0D+00
      AY2=0.0D+00
      AXY=0.0D+00
      DO 1 I=1,J8
         AX=AX+X(I)
         AY=AY+Y(I)
         AXY=AXY+X(I)*Y(I)
         AX2=AX2+X(I)*X(I)
         AY2=AY2+Y(I)*Y(I)
    1 CONTINUE
      CONST=(AY*AX2-AX*AXY)/(J8*AX2-AX*AX)
      SLOPE=((J8*AXY-AX*AY)/(J8*AX2-AX*AX))
      CORR=(J8*AXY-AX*AY)/(DSQRT((J8*AX2-AX*AX)*(J8*AY2-AY*AY)))
      WRITE(*,2)CONST,SLOPE,CORR
      RETURN
    2 FORMAT('   INTERCEPT=',D12.4,'    SLOPE=',D10.3,
     &'    CORRELATION=',F8.4)
      END
C
C*****************************************************************
C
C Calculates time required to transform to a given volume fraction
C
      SUBROUTINE ATIM(SV,BI,TIME,ALP,VOL,XBARN,XAGA,XGAG,VMAX,WSI,
     &                XBAR,XAGA1,XGAG1)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION D(70),E(70)
C
      ADUM=1.0D-8
C
C This statement determines the entire accuracy at high cooling rates
C
      IF(VOL .LE. 1.0D-08) GOTO 4
C
      DTIME=TIME
      J=0
      I=0
C
C ########################## v
C
    3 CALL AVOLFW2(AREA,ANS,VF,SV,BI,DTIME,ALP,DVOL,XBARN,XAGA,
     &         XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)         
C     
C ########################## ^
C
      I=I+1
      D(I)=DVOL/VOL
      E(I)=DABS(D(I)-1.0D+00)
      E1=DABS(1.0/D(I)-1.0D+00)
      IF(DABS(D(I)) .GT. (1.0D+00+1.0D-11) ) GOTO 13
      IF(DABS(D(I)) .LT. (1.0D+00-1.0D-11) ) GOTO 13
      TIME=DTIME
      GOTO 2
   13 IF(I .GT. 1 .AND. J .LT. 1)GOTO 10
      GOTO 11
   10 IF(E(I) .GT. E(I-1))GOTO 12
      IF(E1 .GT. E(I-1)) GOTO 12
      GOTO 11
   12 DTIME=DTIME*0.9D+00
C
C########################### v
C
      CALL AVOLFW2(AREA,ANS,VF,SV,BI,DTIME,ALP,DVOL,XBARN,
     &             XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)               
C
C########################### ^
C
      IF((DVOL-VOL) .GT. 0.0D+00) GOTO 12
      J=J+1
      GOTO 3
C
C Newton's Iterative method
C
   11 DTIM1=DTIME-ADUM
      DTIM2=DTIME+ADUM
C
C########################### v
C
      CALL AVOLFW2(AREA,ANS,VF,SV,BI,DTIM1,ALP,DVOL1,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)        
C
C########################### ^
C
C########################### v
C
      CALL AVOLFW2(AREA,ANS,VF,SV,BI,DTIM2,ALP,DVOL2,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)        
C
C########################### ^
C
      ADUM2=((DVOL-VOL)*(-2.0D+00*ADUM))/(DVOL1-DVOL2)
      IF(DABS(ADUM2) .GT. DTIME)GOTO 14
      DTIME=DTIME-ADUM2
      GOTO 3
   14 DTIME=1.5D+00*DTIME
      GOTO 3
C
C VOL = volume fraction / equilibrium volume fraction
C
    4 TIME=0.0D+00
C
    2 RETURN
      END
C
C*****************************************************************
C
C Calculates time required to transform to a given volume fraction
C
      SUBROUTINE ATIMA(SV,BI,TIME,ALP,VOL,XBARN,XAGA,XGAG,VMAX,WSI
     &,XBAR,XAGA1,XGAG1)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION D(5000),E(5000)
C
      ADUM=1.0D-8
C
C This statement determines the entire accuracy at high cooling rates
C
      IF(VOL .LE. 1.0D-08) GOTO 4
C
      DTIME=TIME
      J=0
      I=0
C
C ########################## v
C
    3 CALL AVOLFW1(AREA,ANS,VF,SV,BI,DTIME,ALP,DVOL,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)      
C     
C ########################## ^
C
      I=I+1
      D(I)=DVOL/VOL
      E(I)=DABS(D(I)-1.0D+00)
      E1=DABS(1.0/D(I)-1.0D+00)
      IF(DABS(D(I)) .GT. (1.0D+00+1.0D-11) ) GOTO 13
      IF(DABS(D(I)) .LT. (1.0D+00-1.0D-11) ) GOTO 13
      TIME=DTIME
      GOTO 2
   13 IF(I .GT. 1 .AND. J .LT. 1)GOTO 10
      GOTO 11
   10 IF(E(I) .GT. E(I-1))GOTO 12
      IF(E1 .GT. E(I-1)) GOTO 12
      GOTO 11
   12 DTIME=DTIME*0.9D+00
C
C ########################### v
C
      CALL AVOLFW1(AREA,ANS,VF,SV,BI,DTIME,ALP,DVOL,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)         
C   
C ########################### ^
C
      IF((DVOL-VOL) .GT. 0.0D+00) GOTO 12
      J=J+1
      GOTO 3
C
C Newton's Iteritive method
C
   11 DTIM1=DTIME-ADUM
      DTIM2=DTIME+ADUM
C
C########################### v
C
      CALL AVOLFW1(AREA,ANS,VF,SV,BI,DTIM1,ALP,DVOL1,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)
C
C########################### ^
C
C########################### v
C
      CALL AVOLFW1(AREA,ANS,VF,SV,BI,DTIM2,ALP,DVOL2,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)        
C                
C########################### ^
C
      ADUM2=((DVOL-VOL)*(-2.0D+00*ADUM))/(DVOL1-DVOL2)
      IF(DABS(ADUM2) .GT. DTIME)GOTO 14
      DTIME=DTIME-ADUM2
      GOTO 3
   14 DTIME=1.5D+00*DTIME
      GOTO 3
C
C VOL = volume fraction / equilibrium volume fraction
C
    4 TIME=0.0D+00
    2 RETURN
      END
C
C***********************************************************************
C
C FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN AUSTENITE
C
      DOUBLE PRECISION FUNCTION CG(X,T,W,R)
      DOUBLE PRECISION J,DG,DUMMY,T,R,W,X
C
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DUMMY=5*DLOG((1-2*X)/X)+6*W/(R*T)+((38575.0) -
     &      (13.48)*T)/(R*T)
      CG=DUMMY+DLOG(((DG-1+3*X)/(DG+1-3*X))**6)
      RETURN
      END
C
C***********************************************************************
C
C FUNCTION GIVING DIFFERENTIAL OF LN(ACTIVITY) OF CARBON IN AUSTENITE, L
C DIFFERENTIAL IS WITH RESPECT TO X
C
      DOUBLE PRECISION FUNCTION DCG(X,T,W,R)
      DOUBLE PRECISION J,DG,DDG,X,T,W,R
C
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DDG=(0.5/DG)*(-2-4*J+2*X+16*J*X)
      DCG=-((10/(1-2*X))+(5/X))+6*((DDG+3)/(DG-1+3*X)-
     &     (DDG-3)/(DG+1-3*X))
      RETURN
      END
C
C***********************************************************************
C
C DAFEG DAFEG DIFFERENTIAL OF LN ACTIVITY OF IRON IN GAMMA
C
      DOUBLE PRECISION FUNCTION DAFEG(XEQ,AJ)
      DOUBLE PRECISION ETEQ,ETEQ2,DEQ,XEQ,AJ
C
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      ETEQ=5*((1/(XEQ-1))+2/(1-2*XEQ))
      ETEQ2=6*((4*AJ-1-(0.5/DEQ)*(-2-4*AJ+2*XEQ+16*XEQ*AJ))
     &/(1-2*AJ+(4*AJ-1)*XEQ-DEQ))+6*(4*AJ/(2*AJ*(2*XEQ-1)))
      DAFEG=ETEQ+ETEQ2
      RETURN
      END
C
C*****************************************************************
C     
      DOUBLE PRECISION FUNCTION ENERGY(T,T10,T20)
      DOUBLE PRECISION T,T10,T20,F,T7
C
      T7=T-100*T20
      IF (T7 .LT. 300) GOTO 1
      IF (T7 .LT. 700) GOTO 2
      IF (T7 .LT. 940) GOTO 3
      F=-8.88909+0.26557*(T7-1140)-1.04923D-3*((T7-1140)**2)
      F=F+2.70013D-6*((T7-1140)**3)-3.58434D-9*((T7-1140)**4)
      GOTO 4
    1 F=1.38*T7-1499
      GOTO 4
    2 F=1.65786*T7-1581
      GOTO 4
    3 F=1.30089*T7-1331
    4 ENERGY=(141.0*T10 + F)*4.187
      RETURN
      END
C
C*****************************************************************
C
C Finds approximate temperature at which paraequilibrium ferrite
C formation first becomes possible
C H. K. D. H. Bhadeshia, 6 July 1994
C
      SUBROUTINE HIGH(C,W,T10,T20,HIGHT,ICI)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION C(8)
C
      X1=C(1)
      XA=0.001D+00
      R=8.31432D+00
      W1=48570.0D+00
      H=38575.0D+00
      S=13.48D+00
      XEQ=0.05D+00
      IC=0
C
C     Modified following line from "DO 1 I=1183,973,-1" to
C     "DO 1 I=1183,893,-1", by K. Ichikawa on 31 Oct., 1994 
C     and modified again to "DO 1 I=1183,879,-1".
     
      DO 1 I=1183,473,-1
         KTEMP=I
         T=KTEMP
         IF (T .LE. 1000.0) GOTO 20
         H1=105525.0D+00
         S1=45.34521D+00
         GOTO 19
   20    H1=111918.0D+00
         S1=51.44D+00
   19    F=ENERGY(T,T10,T20)
         AJ=1.0D+00-DEXP(-W/(R*T))
         AJ1=1.0D+00-DEXP(-W1/(R*T))
   51    TEQ=R*T*AFEG(XEQ,AJ)-F
         IF (DABS(TEQ) .LT. 1.0) GOTO 52
         ETEQ=DAFEG(XEQ,AJ)*R*T
         XEQ=XEQ-TEQ/ETEQ
C
C Modified by K.Ichikawa until ***
C
         IC=IC+1
         IF (IC .GE. 1000000) GOTO 220
C ***
         GOTO 51
   52    IF(XEQ .GE. X1) THEN
            WRITE(*,53)
            HIGHT=KTEMP-273.15D+00
            GOTO 222
         ENDIF
    1 CONTINUE
      IF(XEQ .LT. X1) THEN
         HIGHT=0.0D+00
      ENDIF
  222 RETURN
  220 WRITE(*,221)
      ICI=2
      GOTO 222
   53 FORMAT(5X,'*** Alloy cannot transform to ferrite',
     &          ' at temperature as low as 473 K  ***'//)
  221 FORMAT(5X,'*** Calculation of XEQ could not be',
     &          ' completed correctly  ***'//)
      END
C
C*****************************************************************
C
      SUBROUTINE ISONUC(TIME3,SV,BI,ALP,XBARN,XAGA,XGAG,A1,WSI,VMAX
     &,XBAR,XAGA1,XGAG1)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION A(200)
C
      J2=50
      TINC=TIME3/J2
      TIME=0.0D+00
      SUM=0.0D+00
C
      DO 1 I=1,J2
   12    TIME=TIME+TINC
C
C######################## v
C
         CALL AVOLFW2(AREA,ANS,VF,SV,BI,TIME,ALP,VOL,XBARN,
     &                XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)
C
C######################## ^
C
         IF(I .EQ. 1 .AND. AREA .GT. A1) GOTO 10
         IF(TIME .GT. TIME3) GOTO 2
         IF(AREA .GT. A1) GOTO 2
         GOTO 11
   10    TINC=TINC/2000.0
         TIME=0.0D+00
         AREA=0.0
         GOTO 12
   11    A(I)=(1.0D+00-AREA)*TINC*BI
         SUM=SUM+A(I)
    1 CONTINUE
    2 CONTINUE
C   2 WRITE(*,3) SUM        
      IF(SUM .EQ. 0.0) GOTO 9
      DALPHA=(2.0D+00/(3.0D+00*SV*SUM))**0.333333
      DGAMMA=2.0D+00/(SV)
C     WRITE(*,8)DALPHA,DGAMMA
    9 RETURN
    3 FORMAT(20X,' Number of particles per unit area =',D12.4)
    8 FORMAT(20X,' DALPHA =',D12.4,' m   DGAMMA = ',D12.4,' m')
      END
C
C*****************************************************************
C
      SUBROUTINE ISONUCA(TIME3,SV,BI,ALP,XBARN,XAGA,XGAG,A1,WSI,VMAX,XBAR
     &,XAGA1,XGAG1)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION A(200)
C
      J2=50
      TINC=TIME3/J2
      TIME=0.0D+00
      SUM=0.0D+00
      DO 1 I=1,J2
   12    TIME=TIME+TINC
C
C######################## v
C
                CALL AVOLFW1(AREA,ANS,VF,SV,BI,TIME,ALP,VOL,XBARN,
     &         XAGA,XGAG,WSI,VMAX,XBAR,XAGA1,XGAG1)         
C
C######################## ^
C
         IF(I .EQ. 1 .AND. AREA .GT. A1) GOTO 10
         IF(TIME .GT. TIME3) GOTO 2
         IF(AREA .GT. A1) GOTO 2
         GOTO 11
   10    TINC=TINC/2000.0
         TIME=0.0D+00
         AREA=0.0
         GOTO 12
   11    A(I)=(1.0D+00-AREA)*TINC*BI
         SUM=SUM+A(I)
    1 CONTINUE
    2 CONTINUE
C   2 WRITE(*,3) SUM
    9 RETURN
    3 FORMAT(20X,' Number of particles per unit area =',D12.4)
      END
C
C**************************************************************************
C
C KELVIN = temperature in kelvin
C H, K = Planck constant, Boltzmann constant respectively
C Q = activation energy for self-diffusion of iron
C GSTAR = activation energy for grain boundary nucleation
C NS is the number of nucleation sites per unit area of boundary
C BI is the Grain boundary nucleation rate per second, per square meter
C SV is deleted from the equation of BI=(NS*K*KELVINE/H)*DEXP(-ARG),
C because Sally found the mistake of that on DUMM9 Jan. 1995. By K. Ichikawa.
C CTEMP is temperature in centigrade
C GV is gibbs free energy change per unit volume, for nucleation, J/m**3
C SIGMA is an interfacial energy per square meter
C
      SUBROUTINE NUC(SITE,GSTAR,CTEMP,GV,BI,DUMMY,FIDDLE,SIGMA)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
C
      KELVIN=CTEMP+273.15D+00
      K=1.38062D-23
      H=6.6262D-34
      GSTAR=SIGMA**3.0D+00*DUMMY/(GV*GV)
C
C calculate number of sites pers unit area by dividing by the area
C  of an atom, taking width as 2.5d-10 metres. SITE is a fiddle factor
C
      NS=SITE/((2.5D-10)**2)
      Q=FIDDLE*2.4D+05/6.02217D+23
      ARG=(GSTAR+Q)/(K*KELVIN)
      CALL AARG(ARG)
      BI=(NS*K*KELVIN/H)*DEXP(-ARG)
    2 RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE OMEGA(C,W,XBAR,T10,T20,J)
C
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
C
      DOUBLE PRECISION C(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XBAR
      INTEGER U,J
C
      B3=0.0D+00
      IF(J .EQ. 1) GOTO 2
      C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=C(1)/12.0115D+00
      C(2)=C(2)/28.09D+00
      C(3)=C(3)/54.94D+00
      C(4)=C(4)/58.71D+00
      C(5)=C(5)/95.94D+00
      C(6)=C(6)/52.0D+00
      C(7)=C(7)/50.94D+00
    2 B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 U=2,7
         Y(U)=C(U)/C(8)
  107 CONTINUE
      DO 106 U=1,8
         C(U)=C(U)/B1
  106 CONTINUE
      XBAR=C(1)
C
C Following 2 lines are eliminated after consulting Harry.
C 25 July 1996. K. Ichikawa
C
C      XBAR=DINT(10000.0D+00*XBAR)
C      XBAR=XBAR/10000
C
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
      P(2)=2013.0341+763.8167*C(2)+45802.87*C(2)**2-280061.63*C(2)**3
     &+3.864D+06*C(2)**4-2.4233D+07*C(2)**5+6.9547D+07*C(2)**6
      P(3)=2012.067-1764.095*C(3)+6287.52*C(3)**2-21647.96*C(3)**3-
     &2.0119D+06*C(3)**4+3.1716D+07*C(3)**5-1.3885D+08*C(3)**6
      P(4)=2006.8017+2330.2424*C(4)-54915.32*C(4)**2+1.6216D+06*C(4)**3
     &-2.4968D+07*C(4)**4+1.8838D+08*C(4)**5-5.5531D+08*C(4)**6
      P(5)=2006.834-2997.314*C(5)-37906.61*C(5)**2+1.0328D+06*C(5)**3
     &-1.3306D+07*C(5)**4+8.411D+07*C(5)**5-2.0826D+08*C(5)**6
      P(6)=2012.367-9224.2655*C(6)+33657.8*C(6)**2-566827.83*C(6)**3
     &+8.5676D+06*C(6)**4-6.7482D+07*C(6)**5 +2.0837D+08*C(6)**6
      P(7)=2011.9996-6247.9118*C(7)+5411.7566*C(7)**2
     &+250118.1085*C(7)**3-4.1676D+06*C(7)**4
      DO 108 U=2,7
         B3=B3+P(U)*Y(U)
         B2=B2+Y(U)
  108 CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
  455 W=8054.0
  456 CONTINUE
      RETURN
      END
C
C*************************************************************************
C
      SUBROUTINE TIM(CULRAT1,TIME,AB2,LOWT,CURR,VOLT,SPEED,TINT,J)
C
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
C
C Find constants for particular welding process
C
      CALL HFLOW(J,C1,C2,EFF)
C
C Find cooling rate at intermediate temperature T=(HIGHT-LOWT)/2
C Units of cooling rate in K/s
C
      T = LOWT 
      CULRAT1 = CULRAT(T,TINT,C1,C2,CURR,VOLT,EFF,SPEED)
C
C Find time to cool between HIGHT and LOWT (units of time = s)
C
      TIME=(AB2)/CULRAT1
      RETURN
      END
C
C***********************************************************************
C
C        SUBROUTINE TRAPE
C
C        PURPOSE
C           TO COMPUTE THE VECTOR OF INTEGRAL VALUES FOR A GIVEN
C           GENERAL TABLE OF ARGUMENT AND FUNCTION VALUES.
C
C        USAGE
C             CALL TRAPE (X,Y,Z,NDIM)
C
C        DESCRIPTION OF PARAMETERS
C           X      - DOUBLE PRECISION INPUT VECTOR OF ARGUMENT VALUES.
C           Y      - DOUBLE PRECISION INPUT VECTOR OF FUNCTION VALUES.
C           Z      - THE RESULTING DP. VECTOR OF INTEGRAL VALUES. Z MAY BE
C                    IDENTICAL WITH X OR Y.
C           NDIM   - THE DIMENSION OF VECTORS X,Y,Z. NDIM MAX. 1000
C
C        REMARKS
C           NO ACTION IN CASE NDIM LESS THAN 1.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           BEGINNING WITH Z(1)=0, EVALUATION OF VECTOR Z IS DONE BY
C           MEANS OF TRAPEZOIDAL RULE (SECOND ORDER FORMULA).
C           FOR REFERENCE, SEE
C           F.B.HILDEBRAND, INTRODUCTION TO NUMERICAL ANALYSIS,
C           MCGRAW-HILL, NEW YORK/TORONTO/LONDON, 1956, PP.75.
C
C     ..................................................................
C
      SUBROUTINE TRAPE(X,Y,ANS,NDIM)
C
      DOUBLE PRECISION X(1000),Y(1000),AZ(1000)
      DOUBLE PRECISION SUM1,SUM2,ANS
C
      SUM2=0.D+00
      IF(NDIM-1)4,3,1
C
C     INTEGRATION LOOP
C
    1 DO 2 I=2,NDIM
         SUM1=SUM2
         SUM2=SUM2+.5D+00*(X(I)-X(I-1))*(Y(I)+Y(I-1))
    2 AZ(I-1)=SUM1
    3 AZ(NDIM)=SUM2
      ANS=SUM2
    4 RETURN
      END
C
C*******************************************************************
C
      SUBROUTINE GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
C
      IMPLICIT REAL*8(A-H,K-Z)
C
C CALCULATION OF THE OPTIMUM NUCLEUS C CONTENT AND ACTIVITY OF C IN
C FERRITE NUCLEUS
C
C Estimate a value for X
C February 1991
C
       X=6.3998D-07*T-3.027D-04
       IF(X .LT. 0.1D-07)X=1.0D-08
       IF(X .GT. 0.32D-03)X=0.32D-03
C
C Estimation complete
C
      AJ1=1.0D+00-DEXP(-W1/(R*T))
    1 D1=DSQRT(9.0D+00-6.0D+00*X*(2.0D+00*AJ1+3.0D+00)
     &+(9.0D+00+16.0D+00*AJ1)*X*X)
      B1=((D1-3.0D+00+5.0D+00*X)/(D1+3.0D+00-5.0D+00*X))
      B2=((3.0D+00-4.0D+00*X)/X)
      B3=(H1 - S1*T + 4.0D+00*W1)/(R*T)
C
      A1=DLOG(B1*B1*B1*B1*B2*B2*B2)+B3
C
      IF(X .GT. 1.0D-08) THEN
         A1FE=DLOG(1.0D+00-X)
      ELSE
         A1FE=X
      ENDIF
C
      TEST=F+R*T*(A1FE-AFE) - R*T*(A1-A)
C
C Newton iteration
C
      IF (DABS(TEST) .GT. 10.0) THEN
         DA1=(3.0D+00*X/(3.0D+00-4.0D+00*X))*(
     &   (4.0D+00*X-3.0D+00)/(X*X)-4.0D+00/X)
         DA2=(0.5D+00/D1)*(-12.0D+00*AJ1-18.0D+00+
     &   18.0D+00*X+32.0D+00*AJ1*X)
         DA2=4.0D+00*(((DA2+5.0D+00)/(D1-3.0D+00+5.0D+00*X))
     &   -((DA2-5.0D+00)/(D1+3.0D+00-5.0D+00*X)))
         DA1=DA1+DA2
         DA1FE=1.0D+00/(X-1.0D+00)
         ERROR=TEST/(R*T*(DA1FE-DA1))
         IF (ERROR .GT. X) ERROR = 0.3D+00*X
         X=DABS(X-ERROR)
         GOTO 1
      ENDIF
C
C End of iteration
C
      RETURN
      END
C
C************************************************************************
C
C FUNCTION GIVING THE EQUILIBRIUM MOL.FRAC. CARBON IN ALPHA
C TEMP READ IN AS CENTIGRADE
C BASED ON MY PAPER ON FIRST ORDER QUASICHEMICAL THEROY,METSCI
C
      DOUBLE PRECISION FUNCTION XALPH(T)
      DOUBLE PRECISION T,CTEMP
C
      CTEMP=T/900.0D+00
      XALPH=0.1528D-02-0.8816D-02*CTEMP+0.2450D-01*CTEMP*CTEMP
     &  -0.2417D-01*CTEMP*CTEMP*CTEMP +
     &   0.6966D-02*CTEMP*CTEMP*CTEMP*CTEMP
      RETURN
      END
C
C********************************************************************
C
      SUBROUTINE HFLOW(J,C1,C2,EFF)
      DOUBLE PRECISION C1,C2,EFF
      INTEGER J
      IF(J .EQ. 2) GOTO 2
      IF(J .EQ. 3) GOTO 3
      IF(J .EQ. 4) GOTO 4
      IF(J .EQ. 5) GOTO 5
      IF(J .EQ. 6) GOTO 6
      IF(J .EQ. 7) GOTO 7
      IF(J .EQ. 8) GOTO 8
      IF(J .EQ. 9) GOTO 9
      IF(J .EQ. 10) GOTO 10
      IF(J .EQ. 11) GOTO 11
      IF(J .EQ. 12) GOTO 12
      IF(J .EQ. 13) GOTO 13
      IF(J .EQ. 14) GOTO 14
      IF(J .EQ. 15) GOTO 15
      IF(J .EQ. 16) GOTO 16
C  Manual Metal Arc
      C1=1325.0
      C2=1.60
      EFF=0.775
      GOTO 22
C
C  Metal Cored Wire, CO2 shielding; SUSAN PARK
    2 C1=4.452
      C2=2.411
      EFF=0.9
      GOTO 22
C
C  Metal Cored Wire, Fogon 20 shielding; SUSAN PARK
    3 C1=28.535
      C2=2.064
      EFF=0.9
      GOTO 22
C
C  Tandem submerged arc
    4 C1= 1.076
      C2= 2.6798
      EFF=0.9
      GOTO 22
C
C  Submerged Arc
    5 C1=4359.0
      C2=1.51
      EFF=0.9
      GOTO 22
C
C  Vertical-Up MMA
    6 C1=1125.0
      C2=1.7
      EFF=0.775
      GOTO 22
C
C  Bead on Plate FCAW Chakravati et al., 1985, Metal Constr., 17, p178R
    7 C1=206.1
      C2=2.0
      EFF=0.700
      GOTO 22
C
C  Bead on Plate Self-Sheilded  Chakravati et al.,1985, Metal Constr., 17, p178R
    8 C1=173.4
      C2=2.0
      EFF=0.700
      GOTO 22
C
C  Bead on Plate SAW  Chakravati et al., 1985, Metal Constr., 17, p178R
    9 C1=150.2
      C2=2.0
      EFF=0.950
      GOTO 22
C
C  Measured time-temperature Horizontal GMAW data EB, DTRC, NRL
C  Interpass temperature 150 
C  Ed Metzbower
   10 C1=24.013
      C2=2.2619
      EFF=0.755
      GOTO 22
C
C  Measured time-temperature Flat GMAW data EB, DTRC, NRL
C  Interpass temperature 150 
C  Ed Metzbower
   11 C1=23.172
      C2=2.2845
      EFF=0.755
      GOTO 22
C
C  Measured time-temperature Vertical GMAW data EB, DTRC, NRL
C  Interpass temperature 150
C  Ed Metzbower
   12 C1=8.3292
      C2=2.3884
      EFF=0.755
      GOTO 22
C
C  Measured time-temperature Vertical GMAW data EB, DTRC, NRL
C  Interpass temperature 93 C Ed Metzbower
C  At To=93 C1 values are 46.540, 41.806, and 36.104
C  At To=93 C2 values are 2.1996, 2.2217, and 2.1888 for
C  horizontal, flat, and vertical welds. There is only a small difference
C  in dT/dt for low To values.
   13 C1=41.483
      C2=2.2030
      EFF=0.755
      GOTO 22
C
C  Computed time-temperature for laser weld 117000 W 5 mm/s EFF=0.50
C   Philips  data, modified laser.for program, centre of weldment
   14 C1=498.42
      C2=2.0500
      EFF=0.50
      GOTO 22
C
C  Large heat input tandem submerged arc welding (130kJ/cm) 
C  Data obtained by K. Ichikawa, Nippon Steel
   15 C1= 46130.0
      C2= 1.10
      EFF=0.9
      GOTO 22
C
C  Large heat input tandem submerged arc welding (70kJ/cm)
C  Data obtained by K. Ichikawa, Nippon Steel
   16 C1= 258600.0
      C2= 0.864
      EFF=0.9
      GOTO 22
   22 RETURN
      END
C
C*********************************************************************
C     CULRAT CULRAT - FUNCTION TO GET COOLING RATE AT TEMPERATURE T
C
      DOUBLE PRECISION FUNCTION CULRAT
     &                          (T,TINT,C1,C2,CURR,VOLT,EFF,SPEED)    
      IMPLICIT REAL*8 (A-H,O-Z)
C
      CULRAT=(T-TINT)**C2
      CULRAT=CULRAT*C1*SPEED/(CURR*VOLT*EFF)
      RETURN
      END
C
C************************************************************************
C
      SUBROUTINE INITCC(C,CARBON,SI,MN,NI,MO,CR,V)
C
C This SUROUTINE stores the average chemical compositions.
C The function of this program is INITiarizing the Chemical 
C Compositions to the average compositions.
C K. Ichikawa on 5 January, 1995
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION C(8)
C    
      C(1)=CARBON
      C(2)=SI
      C(3)=MN
      C(4)=NI
      C(5)=MO
      C(6)=CR
      C(7)=V
      RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE SCHEIL(CARBON,SI,MN,NI,MO,CR,V,C,I,N)
C
C     Sheil equation is applied in this subroutine by K. Ichikawa.
C     on 12 Jan. 1995.
C     However, g in original Sheil equation is substituted
C     to i/n-1/(2*n).
C     n is a number of division.
C     i means 'i'th  from the g=0.
C
      IMPLICIT REAL*8(A-H,K-M,O-Z), INTEGER(J,I,N)
      DOUBLE PRECISION C(7)
C
      CALL EDC(CARBON,SI,MN,NI,MO,CR,V,KC,KSI,KMN,KNI,KMO,KCR,KV)
      CSC =KC *C(1)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KC -1.00))
      CSSI=KSI*C(2)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KSI-1.00))
      CSMN=KMN*C(3)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KMN-1.00))
      CSNI=KNI*C(4)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KNI-1.00))
      CSMO=KMO*C(5)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KMO-1.00))
      CSCR=KCR*C(6)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KCR-1.00))
      CSV =KV *C(7)*((1.00-((1.0*I/N)-(1.00/(2.00*N))))**(KV -1.00))
      C(1)=C(1)
      C(2)=CSSI
      C(3)=CSMN
      C(4)=CSNI
      C(5)=CSMO
      C(6)=CSCR
      C(7)=CSV
      RETURN
      END
C
C************************************************************************
C
C PROGRAM TO CALCULATE Equivalent Distribution Coefficient
C M=MELTING POINT IN KELVIN, LIQUIDUS TEMPERATURE
C CARBON PARTITIONING IGNORED
C THERMODYNAMIC DATA FROM KIRKALDY AND BAGANIS
C SOLIDIFICATION AS DELTA FERRITE
C NOTICE: Originaly, the name of this subroutine is "HETRO"
C         which is made by Dr. H.K.D.H.Bhadeshia.
C         
C     Modified by K. ICHIKAWA on 12 Jan. 1995.
C     Sinple empirical calculations of Liquidus Temperature.
C
C     LIQUID(Centigrade) is a liquidus temperature and is calculated 
C     by very simple empilical approximation (L. SMRHA: Solidification and 
C     crystallization of steel ingots, SNTL, Prague, 1983 (in Czech)).
C
      SUBROUTINE EDC(CARBON,SI,MN,NI,MO,CR,V,
     &                               KC,KSI,KMN,KNI,KMO,KCR,KV)
C
      IMPLICIT REAL*8(A-H,K-Z)
C
      LIQUID=1537.0-
     &(88.0*CARBON+8.0*SI+5.0*MN+4.0*NI+2.0*MO+1.5*CR+2.0*V)
C 
      M=LIQUID+273.0
      WRITE(*,2) M
      A=4.187D+00
      R=8.3143D+00
      PC =1.0
      PSI=DEXP((3.9*M-8200.0)*A/(R*M))
      PMN=DEXP((3100.0-2.308*M)*A/(R*M))
      PNI=DEXP((-2120.0-0.38*M)*A/(R*M))
      PCR=DEXP((2.19*M-4600.0)*A/(R*M))
      PMO=DEXP((2.29*M-6600.0)*A/(R*M))
      PV =DEXP((-5100.0+2.3*M)*A/(R*M))      
      KC =PC
      KSI=PSI
      KMN=PMN
      KNI=PNI
      KMO=PMO
      KCR=PCR
      KV =PV
      RETURN
    2 FORMAT('   '/
     & ' Assumed Melting Point of Delta Ferrite=',F8.0,' K')
      END
C
C************************************************************************
C
C This SUBROUTINE calculates the value of "n" which is used to apply the
C Scheile's equation. By K. Ichikawa on 20 Jan. 1995
C
      SUBROUTINE DETN(C,SV,DUMM,SITE,LOWT,J,CURR,VOLT,SPEED,TINT,  
     &                CTEMP,VFAW,VFA,VFW,VFBAF,VFRGM,J2,HIGHT,J10,
     &                CTEMPW,IIMP,ISTOP)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION CTEMP(2000),C(8),VFAW(200),VFA(200),VFW(200)
      DOUBLE PRECISION VFBAF(200),VFRGM(200)
C
      WRITE(*,446)
C     WRITE(*,500)
C
C Calculate the Chemical compositions in the most depleted region.
C
      CARBON=C(1)
      CALL EDC(CARBON,SI,MN,NI,MO,CR,V,
     &                            KC,KSI,KMN,KNI,KMO,KCR,KV)
      CC0 =KC* C(1)
      CSI0=KSI*C(2)
      CMN0=KMN*C(3)
      CNI0=KNI*C(4)
      CCR0=KCR*C(5)
      CMO0=KMO*C(6)
      CV0 =KV* C(7)
C 
      C(1)=CC0
      C(2)=CSI0
      C(3)=CMN0
      C(4)=CNI0
      C(5)=CCR0
      C(6)=CMO0
      C(7)=CV0
C
C Calculate the VF in the most depleted region.
C
      CALL MAINCONP(C,SV, DUMM,SITE,LOWT,J,CURR,VOLT,SPEED,
     &              TINT,CTEMP,VFAW,VFA,VFW,VFBAF,VFRGM,J2,HIGHT,
     &           ICI,CARBON,SI,MN,NI,MO,CR,V,VW,CTEMPW,IIMP,ISTOP)
C
C Determin the "n". J10, in the following procedure, means the "n". 
C      
C     J10=1/VF(J2)
      J10=100
      WRITE(*,560) J10
      WRITE(*,446)
      RETURN
  446 FORMAT('--------------------------------------------------',
     &       '--------------------------------',
     &       '------------------------------------')
  560 FORMAT('   '/
     & ' Determination of n.            n =',I5/)
      END
C
C************************************************************************
C
C This SUBROUTINE can calculate the sum of VF which are divided by n.
C by K. Ichikawa on 12 Jan., 1995.
C
C Sum is made by numerically, so that any interpo. method is not 
C adapted here.
C
C JN(I) is a number of calculation for the 'T'th divided segment.
C 
C VFAWI(I), VFAI(I), VFWI(I), VFBAFI(I) and VFRGMI(I) will be 
C used at the DO 110 loop later on.
C
C Total volume fraction of each microstructual component in the weld
C FVFAW  - VF_alpha+W
C FVFA   - VF_alpha
C FVFW   - VF_W
C FVFBAF - VF_bainite+AF
C FVFRGM - VF_Retained gamma+matensitE
C
C Details in each solidification segment:
C VFAWI(I)  - Segment VF_alpha+W VF_alpha
C VFAI(I)   - Segment VF_W
C VFWI(I)   - Segment VF_B+AF
C VFBAFI(I) - Segment VF_RA+?
C VFRGMI(I) - Segment 
C 
      SUBROUTINE SUMF(CTEMPE,VFEAW,VFEA,VFEW,VFEBAF,VFERGM,J10,ISUM)
C
      IMPLICIT REAL*8(A-H,K-M,O-Z), INTEGER(I,J,N)
      DOUBLE PRECISION CTEMPE(200),VFEAW(200),VFAWI(200),VFEA(200)
      DOUBLE PRECISION VFAI(200),VFEW(200),VFWI(200),VFEBAF(200)
      DOUBLE PRECISION VFERGM(200),VFRGMI(200),VFBAFI(200)
C
      VFAWI(1) =VFEAW(1)
      VFAI(1)  =VFEA(1)       
      VFWI(1)  =VFEW(1)
      VFBAFI(1)=VFEBAF(1)
      VFRGMI(1)=VFERGM(1)
C
      DO 10 I=2,J10
C
C SUM for alpha & alpha_W.
C
         VFAWI(I)=VFEAW(I)
         VFEAW(I)=VFEAW(I-1)+VFEAW(I)
C
C SUM for alpha only.
C
         VFAI(I)=VFEA(I)
         VFEA(I)=VFEA(I-1)+VFEA(I)
C
C SUM for alpha_W only.
C
         VFWI(I)=VFEW(I)
         VFEW(I)=VFEW(I-1)+VFEW(I)
C
C SUM for bainite & AF.
C 
         VFBAFI(I)=VFEBAF(I)
         VFEBAF(I)=VFEBAF(I-1)+VFEBAF(I)
C
C SUM for retained austenite & martensite.
C
         VFRGMI(I)=VFERGM(I)
         VFERGM(I)=VFERGM(I-1)+VFERGM(I)
   10 CONTINUE
      FVFAW =VFEAW(J10) /J10
      FVFA  =VFEA(J10)  /J10
      FVFW  =VFEW(J10)  /J10
      FVFBAF=VFEBAF(J10)/J10
      FVFRGM=VFERGM(J10)/J10
C
C Print the final results.
C
      IF (ISUM .EQ. 1) GOTO 70   
      IF (ISUM .EQ. 2) GOTO 80
C
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
C    
   70 WRITE(*,*)'   '
      WRITE(*,*)'   '
      WRITE(*,*)'**************************************************'
      WRITE(*,*)'*         Final Result of Calculations           *'
      WRITE(*,*)'**************************************************' 
      WRITE(*,91)
      WRITE(*,1) FVFAW
      WRITE(*,2) FVFA
      WRITE(*,3) FVFW
      WRITE(*,4) FVFBAF
      WRITE(*,5) FVFRGM   
C     OPEN(20,FILE='resultbeta6')
C     WRITE(20,*)FVFAW,' ',FVFA,' ',FVFW,' ',FVFBAF,' ',FVFRGM   
      WRITE(*,6)
      WRITE(*,61)
      WRITE(*,62)
      DO 110 I=1,J10
         WRITE(6,63)I,VFAWI(I),VFAI(I),VFWI(I),VFBAFI(I),VFRGMI(I)
110   CONTINUE  
      GOTO 100
   80 CONTINUE      
C
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C    
  100 CONTINUE
      RETURN
    1 FORMAT(' Volume fraction of alpha + Widmanstatten = ',F8.5)
    2 FORMAT(' Volume fraction of alpha                 = ',F8.5)
    3 FORMAT(' Volume fraction of Widmanstatten         = ',F8.5)
    4 FORMAT(' Volume fraction of bainite + acicular ferrite  = ',F8.5)
    5 FORMAT(' Volume fraction of retained gamma + martensite = ',F8.5)
    6 FORMAT(//'***************************** Details for each ',
     &      'solidification segment ****************************'/)
   61 FORMAT(' Segment   Volume fraction  Volume fraction',
     &       '  Volume fraction    VF bainite &       VF retained')
   62 FORMAT(' number   alpha & Widmanst.    alpha Fe       Widmans',
     &       'tatten   acicular ferrite  gamma & martensite')
   63 FORMAT(2X,I3,9X,F8.5,9X,F8.5,9X,F8.5,9X,F8.5,11X,F8.5)
   91 FORMAT(//' Total volume fraction of each microstructual', 
     &         ' component in the weld:'/)
   92 FORMAT(//' Total Volume Fraction of Widmanstatten Ferrite'/) 
   93 FORMAT(F7.5,' ',F7.5,' ',F7.5,' ',F7.5)  
      END
C
C************************************************************************
C
C H. K. D. H. Bhadeshia, Diffusivity of carbon in austenite
C Department of Materials Science and Metallurgy
C Pembroke Street, Cambridge CB2 3QZ, U. K.
C 31 March 1987
C
C Program for the calculation of the diffusivity of carbon in austenite
C as a function of carbon concentration and temperature. Also accounts for
C the infulence of C, Mn, Si, Ni, Mo, Cr and V on the activity of carbon
C in austenite, and therefore on the diffusivity of carbon.
C
C WDIFF is the carbon--carbon interaction energy in austenite, not
C allowing for the presence of Mo, Cr or V
C
C diffusion coefficient in cm**/s
C     D=DIFFUSIVITY OF CARBON IN AUSTENITE
C     Z=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      SUBROUTINE DIFFUS(DIF,X,T,WDIFF)
      IMPLICIT REAL*8(A-H,O-Y), INTEGER(I-N,Z)
C
      W=WDIFF
      HH=6.6262D-34
      BOLTZ=1.38062D-23
      Z=12
      A5=1.0D+00
      R=8.31432D+00
      DASH=(BOLTZ*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
      THETA=X/(A5-X)
      ACTIV=CG(X,T,W,R)
      ACTIV=DEXP(ACTIV)
      DACTIV=DCG(X,T,W,R)
      DACTIV=DACTIV*ACTIV
      DACTIV=DACTIV*A5/((A5+THETA)**2)
      SIGMA=A5-DEXP((-(W))/(R*T))
      PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &    (A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
      DIF=DASH*PSI
      RETURN
      END
C
C*******************************************************************
C
C Modified, 30-8-83, accurate XAPLPHA
C Modified, 30-8-83, accurate CAPILLARITY CONSTANT calculation
C CAPCON capillarity constant (normally written as capital gamma)
C XMAXR equlibrium concentration in austenite at plate tip of finite radius
C T      - ABSOLUTE TEMPERATURE, DEGREES KELVIN
C CTEMP  - TEMPERATURE IN DEGREES CENTIGRADE
C XBAR   - MOL FRAC CARBON IN ALLOY
C XMAX   - EQUILIBRIUM MOL FRAC CARBON IN AUSTENITE
C XALPHA - EQUILIBRIUM MOL FRAC OF C IN FERRITE
C
      SUBROUTINE PROGC(VMAX,T,XMAX,WDIFF,W,XBAR)
C
      IMPLICIT REAL*8 (A-H,K-Y), INTEGER(I,J,Z)
      DOUBLE PRECISION DIFF(1000),CARB(1000)
C
      R=8.31432D+00
      RADIUS=0.0
      VMAX=0.0
      M1=0.00
C     WRITE(*,1009)
      CTEMP=T
      T=T+273.00D+00
      II2=0
      XALPHA=XALPH(CTEMP)
C     WRITE(*,1005)T,CTEMP,XBAR,XMAX,XALPHA
      CALL RRAD(RADIUS,XMAX,XALPHA,XBAR,T,R,XMAXR,W)
      DO 999 II=1,1000
         CARB(1)=XBAR
         IF (II .GT. 1) GOTO 1000
         GOTO 1001
 1000    CARB(II)=CARB(II-1)+0.0001D+00
         IF (CARB(II) .GT. XMAXR) GOTO 1002
 1001    X=CARB(II)
         II2=II2+1
         CALL DIFFUS(DIFF(II),X,T,WDIFF)
  999 CONTINUE
 1002 II3=0
      CALL TRAPE(CARB,DIFF,ANS,II2)
      ANS=ANS/(XMAXR-XBAR)
C     WRITE(*,1004)ANS,ERROR
      CALL VEL4(VMAX,ANS,RADIUS,XMAX,XBAR,XALPHA)
C
C     Convert T to in degree C (by K. Ichikawa on 16 Feb 1996)
C
      T=T-273.00D-00     
      RETURN
 1004 FORMAT(' INTEGRAL, XMAXR-XBAR = ',D12.4, '  CM**2/SEC  ',
     &8HERROR = , D12.4)
 1005 FORMAT(' ABSOLUTE TEMPERATURE, DEGREES KELVIN =',F8.1/
     &'           TEMPERATURE IN DEGREES CENTIGRADE =',F8.1/
     &' MOL FRAC CARBON IN ALLOY = ',F8.4/
     &' EQUILIBRIUM MOL FRAC CARBON IN AUSTENITE =',F8.4/
     &' EQUILIBRIUM MOL FRAC OF C IN FERRITE=',D12.4)
 1009 FORMAT('**************************'/5H     )
      END
C
C**********************************************************************
C
C H. K. D. H. Bhadeshia, November 1993
C Diffusion-controlled growth of parabolic cylinders, Trivedi analysis
C
      SUBROUTINE VEL4(VMAX,ANS,RADIUS,XMAX,XBAR,XALPHA)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
C
C     WRITE(*,4)
C
C Numerical method constants.
C If the solution fails to converge, then DD should be changed
C to some smaller value. (April 1994)
C
      I2=1000
      DD=0.002
C
C Modification, 6 November 1993 to allow for the case when XALPHA > XMAXC
C
      IF (XBAR .LE. XALPHA) THEN
         OMEG=0.99D+00
      ELSE
         OMEG=(XMAX-XBAR)/(XMAX-XALPHA)
      ENDIF
C
      P=(1.0D+00/8.0D+00)*(OMEG/(1.0D+00-OMEG))
      VMAX=(ANS*1.0D-04)*P/RADIUS
      VDUM=DD*VMAX
      VMAX=0.5*VMAX
      ANS=ANS*1.0D-04
C     WRITE(*,44)
C
C Warning - the following polynomial has been derived for the
C OMEG range 0.4 to 1 only
C
      RAD=0.2026D+01-0.1917D+01*OMEG-0.8953D+00*OMEG*OMEG
     &    +0.3670D+01*OMEG*OMEG*OMEG
     &     -0.2519D+01*OMEG*OMEG*OMEG*OMEG
      RAD=10.0D+00**RAD
      RAD=RADIUS*RAD
C
      DO 1 I1=1,I2
         PECLET=VMAX*RAD/(2.0D+00*ANS)
         S2=-1.073019925D+00*(DLOG10(PECLET))-0.273767575D+00
         S2=10.0D+00**S2
         PI=3.14159D+00
         DUMMY=(DSQRT(PI*PECLET))*(DEXP(PECLET))*
     &        (DAERFC(DSQRT(PECLET)))*(1.0D+00+(RADIUS/RAD)*OMEG*S2)
         DDD=DABS(100.0D+00*(DUMMY-OMEG))
           IF (DDD .GT. 0.05D+00) GOTO 55
C   
C       Following WRITE Statement was deleted in oreder to avoid the too
C       much display of the OMEGA and so on. That WRITE Statement was moved
C       just before RETURN Statement so that only the final value can be 
C       printed (by K. Ichikawa on 4 Dec. 1995).
C   
C       WRITE(*,2)OMEG,DUMMY,VMAX,PECLET,RAD
C2      FORMAT(7D12.4)
        IF (DDD .LT. 0.01D+00) GOTO 56
   55   VMAX=VMAX+VDUM
    1 CONTINUE
C  56 WRITE(*,2) OMEG,DUMMY,VMAX,PECLET,RAD 
   56 CONTINUE                 
      RETURN
    2 FORMAT('OMEG  DUMMY  VMAX  PECLET  RAD:'/7D12.4)
    4 FORMAT(/' DIFFUSION CONTROLLED GROWTH, TRIVEDI ANALYSIS')
   44 FORMAT('     OMEGA      DUMMY     VMAX(M/S)     PECLET',
     &'     RAD(M)')
      END
C
C***********************************************************************
C
      SUBROUTINE RRAD(RADIUS,XMAX,XALPHA,XBAR,T,R,XMAXR,W)
C
      DOUBLE PRECISION RADIUS,XMAX,XBAR,T,R,SIG,MOLVOL,XMAXR
      DOUBLE PRECISION XALPHA,RAD,OMEG,CAPCON,EPSI,W,DCG
C
      SIG=0.2
C
C SIG=INTERFACIAL ENERGY, JOULES PER METRE SQUARED
C
      MOLVOL=7.0894317D-06*(1.0D+00+3.549D-05*(T-298.0D+00))
C
C MOLVOL = MOLAR VOLUME OF FERRITE
C RADIUS IS THE CRITICAL RADIUS FOR ZERO GROWTH
C RAD IS THE RATIO OF THE ACTUAL RADIUS TO THE CRITICAL RADIUS
C
      EPSI=XMAX*DCG(XMAX,T,W,R)
      CAPCON=(SIG*MOLVOL/(R*T))*((1.0D+00-XMAX)/(XALPHA-XMAX))
     &        /EPSI
      RADIUS=CAPCON*XMAX/(XBAR-XMAX)
      OMEG=(XMAX-XBAR)/(XMAX-XALPHA)
      RAD=0.2026D+01-0.1917D+01*OMEG-0.8953D+00*OMEG*OMEG
     &    +0.3670D+01*OMEG*OMEG*OMEG
     &    -0.2519D+01*OMEG*OMEG*OMEG*OMEG
      RAD=10.00D+00**RAD
      RAD=RADIUS*RAD
      XMAXR=XMAX*(1.0D+00+(CAPCON/RAD))
C     WRITE(*,1)SIG,MOLVOL,RADIUS,XMAXR,CAPCON,EPSI
    1 FORMAT(' INTERFACIAL ENERGY=',F8.4,' JOULES/METERS SQUARED'/
     &' MOLAR VOLUME OF FERRITE(METERS CUBED PER MOL)=',D15.6/
     &' GIBBS THOMPSON CRITICAL RADIUS(METERS)=',D15.6/
     &' EQUILIBRIUM CONC AT PLATE TIP, MOL FRAC, XMAXR=',D15.6/
     &' CAPILLARITY CONSTANT CAPCON,=',D15.6/
     &' NON-IDEALITY PARAMETER EPSI=',D15.6)
      RETURN
      END
C
C*****************************************************************************
C
      DOUBLE PRECISION FUNCTION DAERFC(A)
      DOUBLE PRECISION A,DAERF
      DAERFC=1.0D+00-DAERF(A)
      RETURN
      END
C
C*************************************************************************
C
C  Error function
C
      DOUBLE PRECISION FUNCTION DAERF(Y)
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER            ISW,I
      DOUBLE PRECISION   P(5),Q(3),P1(8),Q1(7),P2(5),Q2(4)
C
C    COEFFICIENTS FOR 0.0 .LE. Y .LT.  .477
C                                  
      DATA               P(1)/-.44422647396874/,
     1                   P(2)/10.731707253648/,
     2                   P(3)/15.915606197771/,
     3                   P(4)/374.81624081284/,
     4                   P(5)/2.5612422994823D-02/
      DATA               Q(1)/17.903143558843/,
     1                   Q(2)/124.82892031581/,
     2                   Q(3)/332.17224470532/
C
C    COEFFICIENTS FOR .477 .LE. Y  .LE. 4.0
C                                  
      DATA               P1(1)/7.2117582508831/,
     1                   P1(2)/43.162227222057/,
     2                   P1(3)/152.98928504694/,
     3                   P1(4)/339.32081673434/,
     4                   P1(5)/451.91895371187/,
     5                   P1(6)/300.45926102016/,
     6                   P1(7)/-1.3686485738272D-07/,
     7                   P1(8)/.56419551747897/
      DATA               Q1(1)/77.000152935229/,
     1                   Q1(2)/277.58544474399/,
     2                   Q1(3)/638.98026446563/,
     3                   Q1(4)/931.35409485061/,
     4                   Q1(5)/790.95092532790/,
     5                   Q1(6)/300.45926095698/,
     6                   Q1(7)/12.782727319629/
C
C    COEFFICIENTS FOR 4.0 .LT. Y
C
      DATA               P2(1)/-.22695659353969/,
     1                   P2(2)/-4.9473091062325D-02/,
     2                   P2(3)/-2.9961070770354D-03/,
     3                   P2(4)/-2.2319245973418D-02/,
     4                   P2(5)/-2.7866130860965D-01/
      DATA               Q2(1)/1.0516751070679/,
     1                   Q2(2)/.19130892610783/,
     2                   Q2(3)/1.0620923052847D-02/,
     3                   Q2(4)/1.9873320181714/
C                                  CONSTANTS
      DATA               XMIN/1.0D-8/,XLARGE/5.6875D0/
      DATA               SSQPI/.56418958354776/
C
C    FIRST EXECUTABLE STATEMENT
C
      X = Y
      ISW = 1
      IF (X.GE.0.0D0) GO TO 5
      ISW = -1
      X = -X
    5 IF (X.LT..477D0) GO TO 10
      IF (X.LE.4.0D0) GO TO 25
      IF (X.LT.XLARGE) GO TO 35
      RES = 1.D0
      GO TO 50
C
C    ABS(Y) .LT. .477, EVALUATE APPROXIMATION FOR ERF
C                                  
   10 IF (X.LT.XMIN) GO TO 20
      XSQ = X*X
      XNUM = P(5)
      DO 15 I=1,4
         XNUM = XNUM*XSQ+P(I)
   15 CONTINUE
      XDEN = ((Q(1)+XSQ)*XSQ+Q(2))*XSQ+Q(3)
      RES = X*XNUM/XDEN
      GO TO 50
   20 RES = X*P(4)/Q(3)
      GO TO 50
C
C    .477 .LE. ABS(Y) .LE. 4.0 , EVALUATE APPROXIMATION FOR ERF
C
   25 XSQ = X*X
      XNUM = P1(7)*X+P1(8)
      XDEN = X+Q1(7)
      DO 30 I=1,6
         XNUM = XNUM*X+P1(I)
         XDEN = XDEN*X+Q1(I)
   30 CONTINUE
      RES = XNUM/XDEN
      GO TO 45
C
C    4.0 .LT. ABS(Y), EVALUATE APPROXIMATION FOR ERF                                  C
   35 XSQ = X*X
      XI = 1.0D0/XSQ
      XNUM = P2(4)*XI+P2(5)
      XDEN = XI+Q2(4)
      DO 40 I=1,3
         XNUM = XNUM*XI+P2(I)
         XDEN = XDEN*XI+Q2(I)
   40 CONTINUE
      RES = (SSQPI+XI*XNUM/XDEN)/X
   45 RES = RES*DEXP(-XSQ)
      RES = 1.0D0-RES
   50 IF (ISW.EQ.-1) RES = -RES
      DAERF = RES
      RETURN
      END
C
C*************************************************************************
C
C Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
C MUCG1.FOR
C calculation of XEQ and GMAX only. Adapted from MUCG46.FOR
C 5 July 1994
C
C Department of Materials Science and Metallurgy, Pembroke St. Cambridge CB2 3QZ
C
C SUBROUTINE for calculations of Widmanstatten Ferrite 
C Originally name of the SUBROUTINE is AG but modified for 
C the Widmanstatten Ferrite
C 
      SUBROUTINE AGWF(C,W,XBAR,T10,T20,KTEMP,XMAX,GMAX)
C
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8)
C
      X1=C(1)
      XA=0.001D+00
      R=8.31432D+00
      W1=48570.0D+00
      H=38575.0D+00
      S=13.48D+00
      XEQ=0.1D+00
      IC=0
C
      T=KTEMP
      CTEMP=T-273.15D+00
C
      IF (T .LE. 1000.0) GOTO 20
      H1=105525.0D+00
      S1=45.34521D+00
      GOTO 19
   20 H1=111918.0D+00
      S1=51.44D+00
C
C Following line 19 is modified from "F=ENERGY(T,T10,T20)" by K. ICHIKAWA 
C on 17th JUly 1995.  
C
   19 F=ENERGY(T,T10,T20)+50.0D+00
      AJ=1.0D+00-DEXP(-W/(R*T))
      AJ1=1.0D+00-DEXP(-W1/(R*T))
   51 TEQ=R*T*AFEG(XEQ,AJ)-F
      IF (DABS(TEQ) .LT. 1.0) GOTO 52
      ETEQ=DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-TEQ/ETEQ
C
C Modified by K. Ichikawa Until ****
C
      IC=IC+1
      IF (IC .GE. 1000000) GOTO 55
C
C ****     
C
      GOTO 51
   52 AEQ=CG(XEQ,T,W,R)
      AFEQ=AFEG(XEQ,AJ)
      A=CG(X1,T,W,R)
      AFE=AFEG(X1,AJ)
      CALL GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      GMAX=R*T*(A1-A)
C
C Following line of "XMAX=XEQ" is added by K. ICHIKAWA
C for the calculation of Widmanstatten Ferrite on 17th 
C July 1995.
C
      XMAX=XEQ
C      
   60 RETURN
   55 WRITE(*,56)
   56 FORMAT(5X,'*** Calculations of XEQ and GMAX could not be'/
     &      10X,' conpleted correctly  ***'//)
      GOTO 60
      END
C
C***********************************************************
C
C
C     K. Ichikawa, University of Cambridge 13 Feb 1996.
C
C     Find the WS, maximum temperature at which Widmanstatten 
C     transformation is possible.
C
C     All temperatures in Kelvine through out this subroutine.
C     TSTART is the temperature at which calculation for Widmanstatten
C     ferrite starts.
C
C     GMAX is maximum free energy change possible during the nucleation. 
C     GAAF is free energy change for growth.
C 
      SUBROUTINE WSTEMP(C,W,GMAX,GAAF,WS,HIGHT)
C
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8)
C
      CALL OMEGA(C,W,XBAR,T10,T20,0)
      CALL WHIGH(C,W,T10,T20,HIGHT,ICI)
      TSTART=HIGHT+273.18
      TSTOP=673.18      
      DT=1.0
      J1=(TSTART-TSTOP)/DT           
      DO 1 I=1,J1
         T=TSTART-DT*I
         GN=-2540.0+3.637*(T-273.18)     
         CALL AG(C,W,XBAR,T10,T20,T,XEQ,GMAX,FPRO,
     &              H,S,H1,S1,W1,F,AJ,AJ1)
         GAAF=FPRO      
         WS=T
         IF(GMAX .LE. GN .AND. GAAF .LE. -50.0) GOTO 10
    1 CONTINUE
   10 RETURN
      END 
C
C***********************************************************
C
      SUBROUTINE WSNUCRATE(WSI,SV,T,GMAX)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)     
C
C Find the nucleation rate of Widman. ferrite.
C
C Based on the equation in (19) the paper, G.I.Rees and H.K.D.H.Bhadeshia,
C Bainite transformation kinetics Part 1 Modified model, Mat. Sci. 
C and Tech. 1992, Vol.8, p985-993. 
C
C U is the volume of a bainitic subunit
C 
      CONSTR=2540.0D+00
      R=8.31432D+00
      U=(10.0D-06)*(10.0D-06)*(0.2D-06)
C     U=(10.0D-06)*(10.0D-06)*(0.2D-06)
C     In the original Harry's paper, K1DASHU = 33.90D+06       
C                                    K2=2.065D+04
C     Modifying to fit the experimental data
C     
      K1DASHUN=5.0D+05
      K2N=     0.155D+06
C
      K1DASHU=5.0D+05
      K1DASHU=K1DASHUN
      K1DASH =U*K1DASHU
      LBAR=2.0/SV
      K1=1.0/(LBAR*K1DASH)
      K2=0.15D+06
      K2=K2N 
C
      WSI=K1*DEXP(-(K2)/(R*T)-K2*GMAX/(CONSTR*R*T))
C
C WSI is the nucleation rate per unit volume here.
C
      WSI=WSI/SV
C
C WSI is now nucleation rate per unit area.
C     WRITE(*,*)'WSI=',WSI
      RETURN
      END
C
C************************************************************************
C
      SUBROUTINE VWMAINCOMP(SV,CURR,VOLT,EFF,SPEED,TINT,
     &T,XMAX,WDIFF,W,XBAR,AB2,VF,J2,CTEMP,J,C,T10,T20,J3,VFW
     &,CTEMPW,HIGHT,IM,WS,JW,TIMED,TTIME,G,VMAX,WSIT)
C
      IMPLICIT REAL*8(A-H,K,L,O-Z), INTEGER(I,J,M,N)
      DOUBLE PRECISION CTEMP(1000),AC(8),BC(8),C(8),VF(1000)
      DOUBLE PRECISION CTEMPW(1000),WSI(1000),VFW(1000),G(1000)
      DOUBLE PRECISION TIMED(1000),TTIME(1000)
C
C Calculations for each cooling rate
C             
      LW=0.0
      WFFT=400.0
      AB2=5.0
      JW=((WS-WFFT)/AB2)+1
C 
      I=IM                
C   
      CTEMPW(I)=CTEMP(I)
      KTEMP=CTEMPW(I)+273.15D+00
C
C        Following CALL AGWF Statement was added to calculate the XMAX
C        by K. Ichikawa on 18 July 1995.
C
      CALL AGWF(C,W,XBAR,T10,T20,KTEMP,XMAX,GMAX)
C
      CTEMPWT=CTEMPW(I)
      CALL TIM(CULRAT1,TIMEI,AB2,CTEMPWT,CURR,VOLT,SPEED,TINT,J)
C
C Following "IF - ENDIF" is not necessary.
C
C         IF (I .EQ. 1) THEN
C         TIME(I)=TIMEI
C         ELSE
C         TIME(I)=TIMEI+TIME(I-1)
C         ENDIF
C
C        CONST1=tan(30degree)
C        C4: empirical constant for obtaining the actual volume fraction of 
C            ferrite.
      A=2.0/((3.0)**(1/2))/SV
      L=2.0*A          
      CONST1=(1/3)**(1/2)
      C4=1.0
C
C Q is half-thickness of allotriomorphic ferrite
C         
      Q=((3.0**(1/2))/2.0)*(1.0-(1.0-VF(J2))**(1/2))
C
      DO 10 ICONP=1,8
         AC(ICONP)=C(ICONP)
         IF (ICONP .EQ. 5 .OR. ICONP .EQ. 6 .OR. ICONP .EQ. 7) THEN
            BC(ICONP)=0.0
         ELSE
            BC(ICONP)=C(ICONP)
         ENDIF
   10 CONTINUE
C
C        Following subroutine "OMEGA" was installed to get "WDIFF".
C
      CALL OMEGA(BC,WDIFF,XBARD,T10D,T20D,0)
      CALL PROGC(VMAX,CTEMPW,XMAX,WDIFF,W,XBAR)
      G(I)=VMAX
C
C     Find the volume fraction of Widmanstatten ferrite,
C
C     BETA=(Thickness of Widman. ferrite)/(Length or Width)
C
      CALL WSNUCRATE(WSIT,SV,KTEMP,GMAX)
      WSI(I)=WSIT
      BETA=0.045
      IF (I .EQ. 1) THEN
         TIMED(I)=TIMEI          
      ELSE
         TIMED(I)=(LN(1.0-VFW(I-1))/(-BETA*(G(I-1)**3)*
     &             WSI(I-1)/4.0))**(1/4)
      ENDIF
      IF (I .EQ. 1) THEN
         TTIME(I)=TIMED(I)
      ELSE
         TTIME(I)=TTIME(I-1)+TIMED(I)
      ENDIF
   20 RETURN
C   5 FORMAT(F5.0,'   ',F7.5)
C   6 FORMAT(//' CTEMP    VW'/)
C  60 FORMAT(//'Widmanstatten Ferrite was impinged with 
C    &Allotriomorphic Ferrite'/)         
C  80 FORMAT(//'Widmanstatten Ferrite was impinged with 
C    &Acicular Ferrite'/)          
      END
C 
C*****************************************************************
C
C Finds approximate temperature at which paraequilibrium ferrite
C formation first becomes possible
C H. K. D. H. Bhadeshia, 6 July 1994
C
C This SUBROUTINE is made for the Widmanstatten Ferrite start temp.
C By K.Ichikawa on 8 Nov. 1995. 
C
      SUBROUTINE WHIGH(C,W,T10,T20,HIGHT,ICI)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION C(8)
C
      X1=C(1)
      XA=0.001D+00
      R=8.31432D+00
      W1=48570.0D+00
      H=38575.0D+00
      S=13.48D+00
      XEQ=0.05D+00
      IC=0
C
C     Modified following line from "DO 1 I=1183,973,-1" to
C     "DO 1 I=1183,893,-1", by K. Ichikawa on 31 Oct., 1994 
C     and modified again to "DO 1 I=1183,879,-1".
C     
      DO 1 I=1183,473,-1
         KTEMP=I
C
         T=KTEMP
C
         IF (T .LE. 1000.0) GOTO 20
         H1=105525.0D+00
         S1=45.34521D+00
         GOTO 19
   20    H1=111918.0D+00
         S1=51.44D+00
C
   19    F=ENERGY(T,T10,T20)+50.0D+00
         AJ=1.0D+00-DEXP(-W/(R*T))
         AJ1=1.0D+00-DEXP(-W1/(R*T))
   51    TEQ=R*T*AFEG(XEQ,AJ)-F
         IF (DABS(TEQ) .LT. 1.0) GOTO 52
         ETEQ=DAFEG(XEQ,AJ)*R*T
         XEQ=XEQ-TEQ/ETEQ
C
C Modified by K.Ichikawa until ***
C
         IC=IC+1
         IF (IC .GE. 1000000) GOTO 220
C
C ***
C
         GOTO 51
   52    IF(XEQ .GE. X1) THEN
            HIGHT=KTEMP-273.15D+00
            GOTO 222
         ENDIF
    1 CONTINUE
      IF(XEQ .LT. X1) THEN
         WRITE(*,53)
         HIGHT=0.0D+00
      ENDIF
  222 RETURN
  220 WRITE(*,221)
      ICI=2
      GOTO 222
   53 FORMAT(5X,'*** Alloy cannot transform to ferrite'/
     &       10X,' at temperature as low as 473 K  ***'//)
  221 FORMAT(5X,'*** Calculation of XEQ could not be'/
     &      10X,' completed correctly  ***'//)
      END
C
C*****************************************************************
C
C SUBROUTINE transfers mole fraction of carbon to wt.% 
C
C K. Ichikawa on 30 Dec. 1995 
C
C Input C, XBARN as mole fraction, Output as wt.%.
C    
      SUBROUTINE MOLTOWT(XBARN,C,CARBON,SI,MN,NI,MO,CR,V)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION C(8)   
C     
      C(1)=XBARN*12.0115D+00
      C(2)=C(2)*28.09D+00
      C(3)=C(3)*54.94D+00
      C(4)=C(4)*58.71D+00
      C(5)=C(5)*95.94D+00
      C(6)=C(6)*52.0D+00
      C(7)=C(7)*50.94D+00  
      C(8)=C(8)*55.84D+00
C
      TOTAL=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
C
      C(1)=C(1)/TOTAL*100.0
      C(2)=C(2)/TOTAL*100.0
      C(3)=C(3)/TOTAL*100.0
      C(4)=C(4)/TOTAL*100.0
      C(5)=C(5)/TOTAL*100.0
      C(6)=C(6)/TOTAL*100.0
      C(7)=C(7)/TOTAL*100.0 
      C(8)=C(8)/TOTAL*100.0
C
      RETURN 
      END
C
C*****************************************************************
C
C Page 68, Book 14, 20-12-1986, H. K. D. H. Bhadeshia
C AREA is the fraction of boundary covered
C TIME is in seconds
C VOl is volume fraction of ferrite/equilibrium vol. frac.
C VF is the actual volume fraction
C ALP is the one-dimensional parabolic thickening rate constant
C ALP1 is the one-dimensional parabolic lengthening rate constant
C BI is the grain boundary face nucleation rate per unit area per unit t
C XGAG, XAGA = equilibrium mol frac of carbon in gamma and alpha respect
C XBAR = average mol frac of carbon in alloy
C SV = austenite grain boundary area per unit volume (1/m)
C
C **********************************************************************
C Written by K. Ichikawa on 11 April 1996.
C 
C WSI=Nucleation Rate of Widmanstatten Ferrite
C G=Growth Rate of Widmanstatten Ferrite
C PHI=y/(G*TIME)
C BETA=Geometrical Factor of Widmanstatten Ferrite (=0.05)
C
      SUBROUTINE AVOLFW2(AREA,ANS,VF,SV,BI,TIME,ALP,
     &                   VOL,XBARN,XAGA,XGAG,WSI,G,XBAR,XAGA1,XGAG1)
C
C For the calcuLation of Widmanstatten Ferrite Volume Fraction.
C Imitated from the SUBROUTINE AVOLF.
C
C Integration until the Widmanstatten Ferrite tip.
C
C K. Ichikawa, 11 April 1996. 
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION FUN(1000),FUN2(1000),M(1000)
      DOUBLE PRECISION y1(1000),y2(1000)
C
      ILOOP=51
      AILOOP=ILOOP*1.0d+00
      ALP1=3.0D+00*ALP
      BETA=0.045D+00
      G1=BETA*G         
      OMEGA=(XGAG-XBARN)/(XGAG-XAGA)
      OMEGAO=(XGAG-XBAR)/(XGAG-XAGA)
C
C Numerical integration using ILOOP-1 slices
C
      DO 1 I=1,ILOOP
          IDUM=I-1
          M(I)=IDUM/(AILOOP-1.0D+00)
          y1(I)=M(I)*ALP*TIME**0.5D+00
          y2(I)=(ALP*TIME**0.5D+00)+
     &           M(I)*(G*TIME-(ALP*TIME**0.5D+00))
C
C ########################################### v
C Modified for the use of calculations of Widmanstatten ferrite.
C Modified again for amending the fundamental mistake.
C 28 June 1996
C
         IF (G .EQ. 0.0D+00) THEN
            ARG =0.5D+00*3.14159D+00*BI*(ALP1**2.0D+00)     
     &        *(TIME**2.0D+00)*
     &        (1.0D+00-(y1(I)/(ALP*TIME**0.5D+00))**4.0D+00)
            ARG2=0.0D+00
         ELSE
            ARG=0.5D+00*3.14159D+00*BI*(ALP1**2.0D+00)     
     &          *(TIME**2.0D+00)*
     &          (1.0D+00-(y1(I)/(ALP*TIME**0.5D+00))**4.0D+00)+
     &          (1.0D+00/3.0D+00)*WSI*(G1)*(TIME**3.0D+00)*
     &          (1.0D+00-(y1(I)/(G*TIME))**3.0D+00)
             ARG2=(1.0D+00/3.0D+00)*WSI*(G1)*(TIME**3.0D+00)*
     &            (1.0D+00-(y2(I)/(G*TIME))**3.0D+00)
         ENDIF
C
C ############################################ ^
C      
         CALL AARG(ARG)
         CALL AARG(ARG2)
         FUN(I) =1.0D+00-DEXP(-ARG )
         FUN2(I)=1.0D+00-DEXP(-ARG2)
    1 CONTINUE    
      CALL TRAPE(y1,FUN, ANS1,ILOOP)
      CALL TRAPE(y2,FUN2,ANS2,ILOOP)
      IF (G*TIME .GT. ALP*TIME**0.5D+00) THEN
         ANS=ANS1+ANS2
      ELSE
         ANS=ANS1
      ENDIF  
      ARG=(2.0D+00*SV/OMEGA)*ANS
      CALL AARG(ARG)
      VOL=1.0D+00-DEXP(-ARG)
      VF=VOL*OMEGAO
      AREA=FUN(1)
      RETURN
      END
C
C*****************************************************************
C
C For the calcuration of Widmanstatten Ferrite Volume Fraction.
C Imitated from the SUBROUTINE AVOLF.
C
C Integration until the Allotriomorphic Ferrite/Austenite Boundary
C
C K. Ichikawa, 11 April 1996. 
C
C Page 68, Book 14, 20-12-1986, H. K. D. H. Bhadeshia
C AREA is the fraction of boundary covered
C TIME is in seconds
C VOl is volume fraction of ferrite/equilibrium vol. frac.
C VF is the actual volume fraction
C ALP is the one-dimensional parabolic thickening rate constant
C ALP1 is the one-dimensional parabolic lengthening rate constant
C BI is the grain boundary face nucleation rate per unit area per unit t
C XGAG, XAGA = equilibrium mol frac of carbon in gamma and alpha respect
C XBAR = average mol frac of carbon in alloy
C SV = austenite grain boundary area per unit volume (1/m)
C **********************************************************************
C Written by K. Ichikawa on 11 April 1996.
C 
C WSI=Nucleation Rate of Widmanstatten Ferrite
C G=Growth Rate of Widmanstatten Ferrite
C PHI=y/(G*TIME)
C BETA=Geometrical Factor of Widmanstatten Ferrite (=0.05)
C
      SUBROUTINE AVOLFW1(AREA,ANS,VF,SV,BI,TIME,ALP,
     &VOL,XBARN,XAGA,XGAG,
     &WSI,G,XBAR,XAGA1,XGAG1)
C
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION FUN(1000),FUN2(1000),M(1000)
      DOUBLE PRECISION y1(1000),y2(1000)
C
      ILOOP=51
      AILOOP=ILOOP*1.0d+00
      ALP1=3.0D+00*ALP
      BETA=0.045D+00
      G1=BETA*G          
      OMEGA=(XGAG-XBARN)/(XGAG-XAGA)
      OMEGAO=(XGAG-XBAR)/(XGAG-XAGA)      
C
C Numerical integration using ILOOP-1 slices
C
      DO 1 I=1,ILOOP
         IDUM=I-1
         M(I)=IDUM/(AILOOP-1.0D+00)
         y1(I)=M(I)*ALP*TIME**0.5D+00
         y2(I)=(ALP*TIME**0.5D+00)+
     &           M(I)*(G*TIME-(ALP*TIME**0.5D+00))
C
C ################################################# v          
C Modified for the use of calculations of Widmanstatten ferrite
C Modified again for amending the fundamental mistake.
C 28 June 1996
C
         IF (G .EQ. 0.0D+00) THEN
            ARG =0.5D+00*3.14159D+00*BI*(ALP1**2.0D+00)     
     &        *(TIME**2.0D+00)*
     &        (1.0D+00-(y1(I)/(ALP*TIME**0.5D+00))**4.0D+00)
            ARG2=0.0D+00
         ELSE
            ARG=0.5D+00*3.14159D+00*BI*(ALP1**2.0D+00)     
     &          *(TIME**2.0D+00)*
     &        (1.0D+00-(y1(I)/(ALP*TIME**0.5D+00))**4.0D+00)+
     &        (1.0D+00/3.0D+00)*WSI*(G1)*(TIME**3.0D+00)*
     &        (1.0D+00-(y1(I)/(G*TIME))**3.0D+00)

            ARG2=(1.0D+00/3.0D+00)*WSI*(G1)*(TIME**3.0D+00)*
     &           (1.0D+00-(y2(I)/(G*TIME))**3.0D+00)
         ENDIF
C
C ################################################## ^
C     
         CALL AARG(ARG)
         CALL AARG(ARG2)
         FUN(I) =1.0D+00-DEXP(-ARG )
         FUN2(I)=1.0D+00-DEXP(-ARG2)
    1 CONTINUE     
      CALL TRAPE(y1,FUN, ANS1,ILOOP)
      CALL TRAPE(y2,FUN2,ANS2,ILOOP)
      ANS=ANS1
      ARG=(2.0D+00*SV/OMEGA)*ANS
      CALL AARG(ARG)
      VOL=1.0D+00-DEXP(-ARG)
      VF=VOL*OMEGAO
      AREA=FUN(1)
      RETURN
      END
C
C*******************************************************************************C
C To calculate the carbon concentration (mole fraction) at the
C  T-zero phase boundary at a specified temperature.
C
C H K D H Bhadeshia, University of Cambridge
C
C XTO is the carbon concentration at the T-zero boundary
C T is the absolute temperature
C STORE is the stored energy in Joules per mole
C H, S, H1, S1, W, W1, F, AJ, AJ1 are thermodynamic quantities
C  defined in subroutines OMEGA, ENERGY and main program
C
      SUBROUTINE AXTO(H,S,H1,S1,XTO,T,W,W1,F,AJ,AJ1,STORE,XBARN,XBAR)
C
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
C
C Initialize J
C
      J=0
C
C Calculate the driving force for the transformation of
C  austenite to ferrite without any change in chemical composition,
C  allowing for Zener ordering
C
C Set the initial value of XTO in order to avoid the error.
C
      XTO=XBARN
      STORE=0.0
C 
C In the following equation of "G=F+400.0", 400(J mol^-1) is 
C a strain energy term due to the shape change.
C See H.K.D.H.Bhadeshia, Bainite in Steels, 1992.
C and H.K.D.H.Bhadeshia, Acta Metallurgica Vol. 29 pp.1117-1130,
C 1981.
C
C Following was previously "G=F+400.0". But it is modified to 
C "G=F".
C
      G=F+350.0
    1 DFTO=FTO1(H,S,XTO,T,W,W1,H1,S1,G,AJ,AJ1)+STORE
C
C Solve for XTO using Newton's iterative method
C
      J=J+1
      IF (DABS(DFTO) .LE. 10.0D+00) GOTO 2
C
C Obtain differential of FTO1 with respect to XTO
C
      G9=G91(XTO,T,W,W1,H1,S1,G,H,S,AJ,AJ1)
C
C Modify original guess of XTO, assuming that convergence occurs
C  within nine iterations
C
      IF (J .GE. 9) GOTO 2
      XTO=XTO-DFTO/G9
      IF (XTO .LE. 1.0D-12) GOTO 3
      GOTO 1
    3 XTO=0.0000D+00
    2 RETURN
      END
C
C*****************************************************************
C
C To calculate the differential of FTO1 with respect to XTO
C  see subroutine FTO1 for description of parameters
C The differentiation is used in order to apply the Newton Iteration
C
C Unpublished work, H. K. D. H. Bhadeshia, University of Cambridge
C  See laboratory book 5B
C
      DOUBLE PRECISION FUNCTION G91(XTO,T,W,W1,H1,S1,F,
     &                              H,S,AJ,AJ1)
C
      DOUBLE PRECISION R,XTO,T,W,W1,H1,S1,F,AJ,AJ1,FD,FD1,DT5
      DOUBLE PRECISION DT6,DZEN1,DZEN2,DZEN3,V1,V2,V3,V4,V5
      DOUBLE PRECISION V6,V7,V8,V9,G1,DZEN6,DZEN7,DZEN8,H,S
C
C MOdified by K. Ichikawa on 30 May 1996.
C R is the universal gas constant
C
      R=8.31432
C
      FD=DSQRT(1.0-2.0*(1.0+2.0*AJ)*XTO+(1.0+8.0*AJ)*XTO*XTO)
      FD1=DSQRT(9.0-6.0*XTO*(2.0*AJ1+3.0)+(9.0+16.0*AJ1)*XTO*XTO)
C
      DT5=28080.0*XTO/(1.0-XTO)
      DT6=T/DT5
C
      IF (DT6 .GT. 1.0) GOTO 48
      IF (DT6 .LT. 0.25) GOTO 47
C
      DZEN1=0.2307+42.7974*DT6-233.8631*(DT6**2)+645.4485*(DT6**3)
     &      -954.3995*(DT6**4)+711.8095*(DT6**5)-211.5136*(DT6**6)
      DZEN2=-2.6702+45.6337*DT6-225.3965*(DT6**2)+567.7112*(DT6**3)
     &      -771.6466*(DT6**4)+538.1778*(DT6**5)-151.3818*(DT6**6)
C
      GOTO 46
C
   47 DZEN2=1.0
      DZEN1=3.295
   46 DZEN3=(((DZEN2*XTO)**2)*(-50898.56)/(1.0-XTO))+DZEN1*T*XTO*
     &      (0.6623741)
C
      DZEN3=DZEN3*4.187
      GOTO 45
   48 DZEN3=0.0
C
   45 V1=FD-1.0+3.0*XTO
      V2=FD+1.0-3.0*XTO
      V3=1.0-2.0*AJ+(4.0*AJ-1.0)*XTO-FD
      V4=2.0*AJ*(2.0*XTO-1.0)
      V5=FD1-3.0+5.0*XTO
      V6=FD1+3.0-5.0*XTO
      V7=(XTO-1.0-2.0*AJ+8.0*XTO*AJ)/FD
      V8=(9.0*XTO-9.0-6.0*AJ1+16.0*AJ1*XTO)/FD1
      V9=H1-(H)-(S1-(S))*T-6.0*W+4.0*W1
      G1=2.0+DLOG(XTO**2)+4.0+DLOG((1.0-XTO)**4)
     &   -10.0-DLOG((1.0-2.0*XTO)**10)
     &   -DLOG((V1/V2)**6)-6.0*((XTO/V1)*(V7+3.0+V1*(3.0-V7)/V2))
     &   +DLOG((V3/V4)**6)-
     &    6.0*((1.0-XTO)/V3)*(4.0*AJ*(1.0-(V3/V4))-V7)
     &   +3.0*(DLOG(3.0-4.0*XTO)-4.0*XTO/(3.0-4.0*XTO))
     &   +DLOG((V5/V6)**4)+4.0*(XTO
     &    /V5)*(V8+5.0+(V5/V6)*(5.0-V8))
      IF (DT6 .GT. 1.0) GOTO 90
      DZEN6=(-3.3948+13.6112*DT6-13.4376*(DT6**2))*T/(28080.0*
     &      ((1.0-XTO)**2))
      DZEN7=(-3.3118+15.7462*DT6-23.2449*(DT6**2))*T/(28080.0*
     &      ((1.0-XTO)**2))
      DZEN8=50898.0*((DZEN2*XTO)**2)/((1.0-XTO)**2)+(-50898.0*(2*
     &    DZEN2*DZEN6*(XTO**2)+2.0*XTO*(DZEN2**2))/(1.0-XTO))+DZEN1
     &    *T*0.6623741+DZEN7*T*XTO*0.6623741
      GOTO 91
   90 DZEN8=0.0
C
   91 G91=V9+R*T*G1+DZEN8-F
C
      RETURN
      END
C
C*****************************************************************
C
C To calculate the free energy change accompanying the transformation
C from austenite to ferrite of the same chemical composition, including
C a Zener ordering term. The latter describes the ordering of carbon
C atoms, that leads the body-centered cubic lattice of ferrite becoming
C body-centered tetragonal. The degree of ordering increases as the
C carbon concentration increases, or as the transformation temperature
C decreases.
C
C H. K. D. H. Bhadeshia
C
C Department of Materials Science and Metallurgy
C Pembroke Street, Cambridge CB2 3QZ, U. K.
C 31 March 1987
C
C MOdified by K. Ichikawa on 30 May 1996.
C
C
C References:
C
C  C. Zener Trans. AIME 167 (1946) 550.
C
C  J. C. Fisher, Metals Transactions 185 (1949) 688-690.
C
C  H. K. D. H. Bhadeshia and D. V. Edmonds
C   Acta Metallurgica 28 (1980) 1265-1273.
C   Also see laboratory book 7.
C
      DOUBLE PRECISION FUNCTION FTO1(H,S,X,T,W,W1,H1,S1,F,AJ,AJ1)
C
      DOUBLE PRECISION R,X,T,T60,ZEN1,ZEN2,ZENER
      DOUBLE PRECISION W,W1,H1,S1,F,AJ,AJ1,D,D11,H,S
C
      D=DSQRT(1.0D+00-2.0D+00*(1.0D+00+2.0D+00*AJ)*X
     &        +(1.0D+00+8.0D+00*AJ)*X*X)
      D11=DSQRT(9.0D+00-6.0D+00*X*(2.0D+00*AJ1+3.0D+00)
     &        +(9.0D+00+16.0D+00*AJ1)*X*X)
C
C Units of T = Kelvin
C
C Calculate T60, which is the value of the actual temperature T
C   divided by the critical temperature beyond which there is no
C   Zener ordering. X is the mole fraction of carbon in the ferrite.
C If T60>1, the transformation occurs above the ordering temperature
C   and Zener ordering does not occur.
C
      T60=T*(1.0D+00-X)/(28080.0D+00*X)
C
      IF (T60 .GT. 1.0) GOTO 18
C
C Calculate the contribution to the free energy change due to
C  Zener ordering.
C
      IF (T60 .GT. 0.25) GOTO 17
C
C Polynomial fit to data in Table 2 of Fischer (1949)
C
      ZEN1=0.2307+42.7974*T60-233.8631*T60*T60+645.4485*T60*T60*T60
     &     -954.3995*(T60**4)+711.8095*(T60**5)-211.5136*(T60**6)
      ZEN2=-2.6702+45.6337*T60-225.3965*T60*T60+567.7112*(T60**3)
     &     -771.6466*(T60**4)+538.1778*(T60**5)-151.3818*(T60**6)
      GOTO 16
C
   17 ZEN2=1.0D+00
      ZEN1=3.295
C
   16 ZENER=(((ZEN2*X)**2)*(-50898.56)/(1.0-X))+ZEN1*T*X*0.6623741
C
C convert from calories per mole to Joules per mole
      ZENER=ZENER*4.187
      GOTO 15
C
   18 ZENER=0.0
C
C Calculation of the Zener ordering term complete
C
C R is the universal gas constant
   15 R=8.31432
C
C calculate the free energy change accompanying the change from
C austenite to ferrite of the same composition, including a Zener
C ordering term. See Bhadeshia and Edmonds (1980)
C
      FTO1=X*R*T*DLOG(X*X)+X*(H1-(H)-(S1-(S))
     &     *T+4.0*W1-6*W)-R*T*(1.0-X)*DLOG((1.0-X)**4)+
     &     5.0*R*T*(1.0-2.0*X)*DLOG(1.0-2.0*X)
     &     -R*T*X*DLOG(((D-1.0+3.0*X)/(D+1.0-3.0*X))**6)
     &     -R*T*(1.0-X)*DLOG(((1.0-2.0*AJ+(4.0
     &     *AJ-1.0)*X-D)/(2.0*AJ*(2.0*X-1.0)))**6)+
     &     3.0*R*T*X*DLOG(3.0-4*X)+R*T*
     &     X*DLOG(((D11-3.0+5.0*X)/(D11+3.0-5.0*X))**4)+
     &     (1.0-X)*F+ZENER
C
       RETURN
       END


